package eduni.inf.nbmodules.pepa.actions;

import eduni.inf.nbmodules.pepa.cookies.StateSpaceCookie;
import eduni.inf.nbmodules.pepa.cookies.SyntaxCheckedCookie;
import eduni.inf.nbmodules.pepa.loader.PepaDataObject;
import eduni.inf.pepa.gui.guiException;
import eduni.inf.pepa.pepa.Peparoni;
import java.io.IOException;
import org.openide.ErrorManager;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CookieAction;



/** Base class for derivations of the state space.
 *
 *  Serves as a hook to @link eduni.inf.pepa.pepa.Peparoni
 *
 *
 * @author N.V.Haenel
 */
public abstract class DeriveAction extends CookieAction {
    
    protected Class[] cookieClasses() {
        return new Class[] {SyntaxCheckedCookie.class };
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
        
    }
    
    protected void performAction(Node[] nodes) {
        // get editor support and data object
        PepaDataObject obj = (PepaDataObject)nodes[0].getCookie(PepaDataObject.class);
        
        try{
            
            // derive the state space of the model
            Peparoni driver = obj.getPeparoni();
            
            driver.setoutoption(this.outputOption());
            driver.derive();
            // derive will throw a gui exception if anything goes wrong
            // preventing the state space cookie from being added.
            
            obj.addCookie(new StateSpaceCookie());
            
            obj.getPrimaryFile().getFileSystem().refresh(true);
            
            
            
        }catch(FileStateInvalidException fsie){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, fsie);
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);
        }catch(guiException ge){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ge);
            
        }
        
    }
    /** Get the desired output option for Peparoni */
    protected abstract int outputOption();
    
    public abstract String getName();
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/pepa/actions/icons/DeriveIcon.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    protected boolean asynchronous() {
        return true;
    }
    
}
