package eduni.inf.nbmodules.pepa.actions;

import eduni.inf.nbmodules.pepa.cookies.ReflectionCookie;
import eduni.inf.nbmodules.pepa.loader.PepaDataObject;
import eduni.inf.pepa.reflector.Reflector;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import org.openide.ErrorManager;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;

/**Action to reflect a solved pepa model into UML
 *
 * @author N.V.HAenel
 */
public class ReflectAction extends CookieAction {
    
    private JFileChooser targetsChooser;
    private JFileChooser outputChooser;
    
    
    protected Class[] cookieClasses() {
        return new Class[] { ReflectionCookie.class };
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
    }
    
    protected void performAction(Node[] nodes) {
        // get data object
        PepaDataObject obj = (PepaDataObject)nodes[0].getCookie(PepaDataObject.class);
        // get reflection cookie
        ReflectionCookie rc = (ReflectionCookie)obj.getCookie(ReflectionCookie.class);
        // get xml results file name
        // will exist becaue this action is enabled
        String xmlresults = obj.getPeparoni().XMLresultsfilename;
        try{
                        
            File searchDirectory = FileUtil.toFile(obj.getPrimaryFile()).getParentFile();
            
            targetsChooser.setCurrentDirectory(searchDirectory);
            
            final List reflectionTargets = rc.getTargets();
            
            targetsChooser.setFileFilter(new FileFilter(){
                public String getDescription(){
                    return "Possible Reflection Targets";
                }
                
                public boolean accept(File f){
                    
                    for(Iterator i = reflectionTargets.iterator();i.hasNext();){
                        FileObject current = (FileObject)i.next();
                        if(current.getNameExt().equals(f.getName()))
                            return true;
                    }
                    return false;
                }
            });
            
            
            int returnVal = targetsChooser.showDialog(null,"Select");
            targetsChooser.resetChoosableFileFilters();            
            File original;
            if(returnVal == JFileChooser.APPROVE_OPTION){
                original = targetsChooser.getSelectedFile();
            }
            else return;
            
            StringBuffer newFile = new StringBuffer(original.getAbsolutePath());
            newFile.insert(newFile.lastIndexOf("."),".performance");
            
            outputChooser.setSelectedFile(new File(newFile.toString()));
            returnVal = outputChooser.showDialog(null,"Reflect");
            
            File reflected;
            if(returnVal == JFileChooser.APPROVE_OPTION){
                reflected = outputChooser.getSelectedFile();
            }
            else return;
            
            
            Reflector.setOriginal(original);
            Reflector.setResults(new File(xmlresults));
            Reflector.setReflected(reflected);
            Reflector.reflect();
                                                
            obj.getPrimaryFile().getFileSystem().refresh(true);
            
        }catch(FileStateInvalidException fsie){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, fsie);
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);
        }
        
    }
    
    public String getName() {
        return NbBundle.getMessage(ReflectAction.class, "Reflect_Action");
    }
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/pepa/actions/icons/ReflectActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx(ReflectActionAction.class);
    }
    
    protected boolean asynchronous() {
        // performAction(Node[]) should run in event thread
        return true;
    }
    
    /** Only make reflectable if the cookie signals that the XML results are available
     *
     */
    protected boolean enable(Node[] nodes) {
        PepaDataObject obj;
        if(nodes.length>0){
            if (( obj = (PepaDataObject)nodes[0].getCookie(PepaDataObject.class))!= null){
                if (obj.getCookie(ReflectionCookie.class)!= null)
                    return ((ReflectionCookie)obj.getCookie(ReflectionCookie.class)).isReflectable();
            }
        }
        return false;
    }
    
    
    protected void initialize() {
        super.initialize();
        
        targetsChooser = new JFileChooser();
        targetsChooser.setDialogTitle(NbBundle.getMessage(ReflectAction.class, "TargetsChooserDialogTitle"));
        targetsChooser.setMultiSelectionEnabled(false);
        targetsChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        outputChooser = new JFileChooser(){
            public void approveSelection(){
                File selected = getSelectedFile();
                if(selected.exists() && (JOptionPane.showConfirmDialog(this,
                "The file "+selected.getName()+
                " already exists, would you like to overwrite it?",
                "Overwrite?",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE) != JOptionPane.OK_OPTION)){return ;}
                else{this.setSelectedFile(selected);}
                super.approveSelection();
            }
        };
        
        outputChooser.setDialogTitle(NbBundle.getMessage(ReflectAction.class, "OutputChooserDialogTitle"));
        outputChooser.setMultiSelectionEnabled(false);
        outputChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        //        putProperty(Action.SHORT_DESCRIPTION, NbBundle.getMessage(ReflectActionAction.class, "HINT_Action"));
    }
    
    
}
