package eduni.inf.nbmodules.pepa.actions;

import eduni.inf.nbmodules.pepa.cookies.ReflectionCookie;
import eduni.inf.nbmodules.pepa.cookies.SolvedCookie;
import eduni.inf.nbmodules.pepa.cookies.StateSpaceCookie;
import eduni.inf.nbmodules.pepa.loader.PepaDataObject;
import eduni.inf.pepa.pepa.Peparoni;
import eduni.inf.pepa.reflector.Reflector;
import eduni.inf.pepa.workbench.XMLresults;
import org.openide.ErrorManager;
import org.openide.filesystems.FileStateInvalidException;

import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;


/** Action to solve the PEPA model
 *
 * @author N.V.Haenel
 */
public abstract class SolveAction extends CookieAction {
    
    protected Class[] cookieClasses() {
        return new Class[] {StateSpaceCookie.class };
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
        // return MODE_ALL;
    }
    
    protected void performAction(Node[] nodes) {
        PepaDataObject obj = (PepaDataObject)nodes[0].getCookie(PepaDataObject.class);
        // the reflection cookie is added here if there are valid reflection targets
        // we need to do this BEFORE solving the model, to ensure
        // that the XML results are generated
        if(obj.reflectionTarget())
            XMLresults.resultsWanted(true);
            Reflector.setOriginal(null);
        // if the solver has been successful,
        // set reflectable to true
        if(this.executeSolver(obj.getPeparoni())){
            ReflectionCookie rc;
            if ((rc = (ReflectionCookie)obj.getCookie(ReflectionCookie.class)) != null)
                rc.setReflectable(true);
            obj.addCookie(new SolvedCookie());
        }
        
        
        try{
            obj.getPrimaryFile().getFileSystem().refresh(true);
        }catch(FileStateInvalidException fsie){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, fsie);                        
        }
        
        
    }
    
    
    protected abstract boolean executeSolver(Peparoni driver);
    
    public String getName() {
        return NbBundle.getMessage(SolveAction.class, "Solve_Action");
    }
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/pepa/actions/SolveActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    protected boolean asynchronous() {
        return true;
    }
    
}
