package eduni.inf.nbmodules.pepa.actions;

import eduni.inf.nbmodules.pepa.editor.PepaEditorSupport;
import eduni.inf.nbmodules.pepa.editor.parser.ParseException;
import eduni.inf.nbmodules.pepa.editor.parser.PepaParser;
import eduni.inf.nbmodules.pepa.loader.PepaDataObject;
import eduni.inf.pepa.gui.guiException;
import eduni.inf.pepa.pepa.Peparoni;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import org.openide.ErrorManager;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;

import org.openide.windows.OutputWriter;


/** Action to check the syntax of a PEPA model.
 *
 * @author N.V.Haenel
 */
public class SyntaxCheckAction extends CookieAction {
    
    protected Class[] cookieClasses() {
        // shuld be syntax check cookie
        return new Class[] { PepaDataObject.class};
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
    }
    
    protected void performAction(Node[] nodes) {
        System.out.println(nodes.length);
        PepaDataObject obj = (PepaDataObject)nodes[0].getCookie(PepaDataObject.class);
        PepaEditorSupport pes = (PepaEditorSupport)nodes[0].getCookie(PepaEditorSupport.class);
        obj.clearAllDialogs();        
        
        try{
            
            OutputWriter ow = PepaIOProvider.getOutputWriter();
            
            
            InputStream input = pes.getInputStream();
            File file = FileUtil.toFile(obj.getPrimaryFile());
            String fileName = file.getName();
            
            
            PepaParser p = new PepaParser(input);
            p.parse();
            
            if(p.getParseExceptions().isEmpty()){
                //ow.println(fileName + ": Syntax Check Successful");
                pes.syntaxChecked();
            }
            else{
                for(Iterator j = p.getParseExceptions().iterator();j.hasNext();){
                    ParseException pe = (ParseException)j.next();
                    ow.println( fileName + ":  ["+ pe.getLineColumn() + "] "+pe.getMessage());
                }
            }
            
            Peparoni.initialise();
            Peparoni driver = obj.getPeparoni();
            driver.setFileNamesNoResultsDirectory(file.getParentFile(), file.getName());
            // load the pepamodel
            driver.loadPepaModel();
            // parse the model
            driver.pepaparser();
                        
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);            
        }catch(guiException ge){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ge);            
        }                        
    }
    
    public String getName() {
        return NbBundle.getMessage(SyntaxCheckAction.class, "SyntaxCheck_Action");
    }
    
    public boolean asynchronous(){
        return false;
    }
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/pepa/actions/icons/CheckSyntaxActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
}
