package eduni.inf.nbmodules.pepa.cookies;

import eduni.inf.nbmodules.pepa.loader.PepaDataObject;
import java.util.List;
import org.openide.filesystems.*;
import org.openide.nodes.Node.Cookie;




/** Reflection cookie signals that the PepaDataObject has
 *  targets for reflection, and that the model has been
 *  solved such that reflection is possible.
 *
 * @author  N.V.Haenel
 */
public class ReflectionCookie extends FileChangeAdapter implements Cookie, FileChangeListener {
    
    private boolean canReflect;
    
    private List targets;
    
    private PepaDataObject pdo;
    
    private String start;
    
    
    public ReflectionCookie(List targets, PepaDataObject pdo){
        this.targets = targets;
        this.canReflect = false;
        this.pdo = pdo;
        pdo.getPrimaryFile().getParent().addFileChangeListener(this);
        String file  = pdo.getPrimaryFile().getNameExt();
        // get the start substring of the lysa file
        start = file.substring(0,file.indexOf('.'));
        this.synchronizeTargets();
        
        
    }
    
    public void synchronizeTargets(){
        // get the other file objects in this folder
        FileObject[] otherObjects = pdo.getPrimaryFile().getParent().getChildren();
        // if they start with the start string and end in either .xmi or .zuml
        // this is a reflection target
        for(int i = 0 ; i< otherObjects.length;i++){
            String current = otherObjects[i].getNameExt();
            if(current.startsWith(start) && (current.endsWith(".xmi") || current.endsWith(".zuml"))){
                if(!targets.contains(otherObjects[i]))
                    targets.add(otherObjects[i]);
            }
        }
    }
    
    public void fileDataCreated(FileEvent fileEvent) {
        FileObject newFileObject = fileEvent.getFile();
        if(!newFileObject.isFolder()){
            String newFile = newFileObject.getNameExt();
            if(newFile.startsWith(start) && (newFile.endsWith(".xmi") || newFile.endsWith(".zuml")))
                targets.add(newFileObject);
        }
    }
    
    public void fileDeleted(FileEvent fileEvent) {
        FileObject newFileObject = fileEvent.getFile();
        targets.remove(newFileObject);
    }
    
    public void fileRenamed(FileRenameEvent fileRenameEvent) {
        FileObject newFileObject = fileRenameEvent.getFile();
        if(!newFileObject.isFolder()){
            targets.remove(newFileObject);
        }
        this.synchronizeTargets();
    }
    
    public void setReflectable(boolean b){
        this.canReflect = b;
    }
    
    
    public boolean isReflectable(){
        return this.canReflect;
    }
    
    
    public List getTargets(){
        return targets;
    }
}









