/*
 *                 Sun Public License Notice
 * 
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 * 
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2000 Sun
 * Microsystems, Inc. All Rights Reserved.
 */

package eduni.inf.nbmodules.pepa.editor;

import eduni.inf.nbmodules.pepa.editor.lang.PepaLanguage;
import java.awt.Color;
import java.awt.Font;
import java.util.HashMap;
import java.util.Map;
import javax.swing.text.Document;
import org.netbeans.editor.*;
import org.netbeans.editor.ext.ExtSyntaxSupport;
import org.netbeans.editor.ext.plain.PlainSyntax;
import org.netbeans.modules.editor.NbEditorKit;
import org.netbeans.modules.lexer.editorbridge.LanguageDescriptor;
import org.netbeans.modules.lexer.editorbridge.LexerLayerInitializer;



/** Editor kit for PEPA.
 *
 * @author N.V.Haenel
 */

public class PepaKit extends NbEditorKit {
    
    static {
        Settings.addInitializer(new LexerLayerInitializer(
            new LanguageDescriptor(PepaLanguage.get(), PepaKit.class,
            "pepa-", createTokenId2ColoringMap()))
        );
    }
 
    public static Map createTokenId2ColoringMap() {
        Map m = new HashMap();

        m.put("comment", new Coloring(new Font("Monospaced", Font.PLAIN, 12),Color.GRAY , null));        
        m.put("operator", new Coloring(null, Color.DARK_GRAY, null)); // Whole category coloring
        m.put("separator", new Coloring(null, Color.DARK_GRAY, null));
        m.put("number", new Coloring(null, Color.blue , null));
        m.put("process-identifier", new Coloring(null, new Color(75,185,75), null));        
        m.put("action-identifier", new Coloring(null, new Color(75,185,185), null));                
        m.put("error", new Coloring(new Font("Monospaced", Font.PLAIN, 12), Color.red, null));
        m.put("incomplete", new Coloring(null, new Color(255, 0, 50), null));
        m.put("keyword", new Coloring(new Font("Monospaced", Font.BOLD, 12), Color.blue, null));
        return m;
    }

    public String getContentType() {
        return "text/x-pepa";
    }

    public Syntax createSyntax(Document doc) {
        return new PlainSyntax();
    }

    public SyntaxSupport createSyntaxSupport(BaseDocument doc) {
        return new ExtSyntaxSupport(doc);
    }

}
