
package eduni.inf.nbmodules.pepa.editor.lang;

import org.netbeans.api.lexer.Lexer;
import org.netbeans.api.lexer.TokenId;
import org.netbeans.spi.lexer.AbstractLanguage;
import org.netbeans.spi.lexer.MatcherFactory;

public class PepaLanguage extends AbstractLanguage {

    /** Maximum lexer state determined from xxxConstants class. */
    static final int MAX_STATE = 3;

    /** Lazily initialized singleton instance of this language. */
    private static PepaLanguage INSTANCE;

    /** @return singleton instance of this language. */
    public static synchronized PepaLanguage get() {
        if (INSTANCE == null)
            INSTANCE = new PepaLanguage();

        return INSTANCE;
    }

    public static final int WHITESPACE_INT = 1;
    public static final int INCOMPLETE_ML_COMMENT_INT = 2;
    public static final int ML_COMMENT_INT = 3;
    public static final int INCOMPLETE_SL_COMMENT_INT = 4;
    public static final int SL_COMMENT_INT = 5;
    public static final int INSIDE_SL_COMMENT_INT = 6;
    public static final int INCOMPLETE_LATEX_COMMENT_INT = 7;
    public static final int LATEX_COMMENT_INT = 8;
    public static final int INSIDE_LATEX_COMMENT_INT = 9;
    public static final int INFTY_INT = 10;
    public static final int TOP_INT = 11;
    public static final int IF_INT = 12;
    public static final int THEN_INT = 13;
    public static final int ELSE_INT = 14;
    public static final int PLUS_INT = 15;
    public static final int MINUS_INT = 16;
    public static final int MUL_INT = 17;
    public static final int DIV_INT = 18;
    public static final int LESS_INT = 19;
    public static final int GREATER_INT = 20;
    public static final int MUL3_INT = 21;
    public static final int PLUS5_INT = 22;
    public static final int PREFIX_INT = 23;
    public static final int EQUALS_INT = 24;
    public static final int SEQCOMP_INT = 25;
    public static final int UNDER_INT = 26;
    public static final int LPAREN_INT = 27;
    public static final int RPAREN_INT = 28;
    public static final int LBRACE_INT = 29;
    public static final int RBRACE_INT = 30;
    public static final int LSQUARE_INT = 31;
    public static final int RSQUARE_INT = 32;
    public static final int SEMI_INT = 33;
    public static final int COMMA_INT = 34;
    public static final int HASH_INT = 35;
    public static final int INTEGER_LITERAL_INT = 36;
    public static final int DECIMAL_LITERAL_INT = 37;
    public static final int FLOATING_POINT_LITERAL_INT = 38;
    public static final int PROCESS_IDENTIFIER_INT = 39;
    public static final int ACTION_IDENTIFIER_INT = 40;
    public static final int LOWER_CASE_LETTER_INT = 41;
    public static final int UPPER_CASE_LETTER_INT = 42;
    public static final int SYMBOL_INT = 43;
    public static final int ML_COMMENT_END_INT = 45;
    public static final int ERROR_INT = 46;
    public static final int IN_SL_COMMENT_INT = 47;
    public static final int IN_LATEX_COMMENT_INT = 48;
    public static final int IN_ML_COMMENT_INT = 49;


    public static final TokenId ACTION_IDENTIFIER = new TokenId("action-identifier", ACTION_IDENTIFIER_INT, new String[]{"action-identifier"});
    public static final TokenId COMMA = new TokenId("comma", COMMA_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher(","));
    public static final TokenId DECIMAL_LITERAL = new TokenId("decimal-literal", DECIMAL_LITERAL_INT);
    public static final TokenId DIV = new TokenId("div", DIV_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("/"));
    public static final TokenId ELSE = new TokenId("else", ELSE_INT, new String[]{"keyword"}, MatcherFactory.createTextCheckMatcher("else"));
    public static final TokenId EQUALS = new TokenId("equals", EQUALS_INT, null, MatcherFactory.createTextCheckMatcher("="));
    public static final TokenId ERROR = new TokenId("error", ERROR_INT, new String[]{"error"});
    public static final TokenId FLOATING_POINT_LITERAL = new TokenId("floating-point-literal", FLOATING_POINT_LITERAL_INT, new String[]{"number"});
    public static final TokenId GREATER = new TokenId("greater", GREATER_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher(">"));
    public static final TokenId HASH = new TokenId("hash", HASH_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher("#"));
    public static final TokenId IF = new TokenId("if", IF_INT, new String[]{"keyword"}, MatcherFactory.createTextCheckMatcher("if"));
    public static final TokenId INCOMPLETE_LATEX_COMMENT = new TokenId("incomplete-latex-comment", INCOMPLETE_LATEX_COMMENT_INT, new String[]{"comment", "incomplete", "error"}, MatcherFactory.createTextCheckMatcher("%"));
    public static final TokenId INCOMPLETE_ML_COMMENT = new TokenId("incomplete-ml-comment", INCOMPLETE_ML_COMMENT_INT, new String[]{"comment", "incomplete", "error"}, MatcherFactory.createTextCheckMatcher("/*"));
    public static final TokenId INCOMPLETE_SL_COMMENT = new TokenId("incomplete-sl-comment", INCOMPLETE_SL_COMMENT_INT, new String[]{"comment", "incomplete", "error"}, MatcherFactory.createTextCheckMatcher("//"));
    public static final TokenId INFTY = new TokenId("infty", INFTY_INT, new String[]{"keyword"}, MatcherFactory.createTextCheckMatcher("infty"));
    public static final TokenId INSIDE_LATEX_COMMENT = new TokenId("inside-latex-comment", INSIDE_LATEX_COMMENT_INT, new String[]{"comment"});
    public static final TokenId INSIDE_SL_COMMENT = new TokenId("inside-sl-comment", INSIDE_SL_COMMENT_INT, new String[]{"comment"});
    public static final TokenId INTEGER_LITERAL = new TokenId("integer-literal", INTEGER_LITERAL_INT, new String[]{"number"});
    public static final TokenId IN_LATEX_COMMENT = new TokenId("in-latex-comment", IN_LATEX_COMMENT_INT, new String[]{"comment"});
    public static final TokenId IN_ML_COMMENT = new TokenId("in-ml-comment", IN_ML_COMMENT_INT, new String[]{"comment"});
    public static final TokenId IN_SL_COMMENT = new TokenId("in-sl-comment", IN_SL_COMMENT_INT, new String[]{"comment"});
    public static final TokenId LATEX_COMMENT = new TokenId("latex-comment", LATEX_COMMENT_INT, new String[]{"comment"});
    public static final TokenId LBRACE = new TokenId("lbrace", LBRACE_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher("{"));
    public static final TokenId LESS = new TokenId("less", LESS_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("<"));
    public static final TokenId LOWER_CASE_LETTER = new TokenId("lower-case-letter", LOWER_CASE_LETTER_INT);
    public static final TokenId LPAREN = new TokenId("lparen", LPAREN_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher("("));
    public static final TokenId LSQUARE = new TokenId("lsquare", LSQUARE_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher("["));
    public static final TokenId MINUS = new TokenId("minus", MINUS_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("-"));
    public static final TokenId ML_COMMENT = new TokenId("ml-comment", ML_COMMENT_INT, new String[]{"comment"});
    public static final TokenId ML_COMMENT_END = new TokenId("ml-comment-end", ML_COMMENT_END_INT, new String[]{"error"}, MatcherFactory.createTextCheckMatcher("*/"));
    public static final TokenId MUL = new TokenId("mul", MUL_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("*"));
    public static final TokenId MUL3 = new TokenId("mul3", MUL3_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("***"));
    public static final TokenId PLUS = new TokenId("plus", PLUS_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("+"));
    public static final TokenId PLUS5 = new TokenId("plus5", PLUS5_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("+++++"));
    public static final TokenId PREFIX = new TokenId("prefix", PREFIX_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("."));
    public static final TokenId PROCESS_IDENTIFIER = new TokenId("process-identifier", PROCESS_IDENTIFIER_INT, new String[]{"process-identifier"});
    public static final TokenId RBRACE = new TokenId("rbrace", RBRACE_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher("}"));
    public static final TokenId RPAREN = new TokenId("rparen", RPAREN_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher(")"));
    public static final TokenId RSQUARE = new TokenId("rsquare", RSQUARE_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher("]"));
    public static final TokenId SEMI = new TokenId("semi", SEMI_INT, new String[]{"separator"}, MatcherFactory.createTextCheckMatcher(";"));
    public static final TokenId SEQCOMP = new TokenId("seqcomp", SEQCOMP_INT, new String[]{"operator"}, MatcherFactory.createTextCheckMatcher("||"));
    public static final TokenId SL_COMMENT = new TokenId("sl-comment", SL_COMMENT_INT, new String[]{"comment"});
    public static final TokenId SYMBOL = new TokenId("symbol", SYMBOL_INT);
    public static final TokenId THEN = new TokenId("then", THEN_INT, new String[]{"keyword"}, MatcherFactory.createTextCheckMatcher("then"));
    public static final TokenId TOP = new TokenId("top", TOP_INT, new String[]{"keyword"}, MatcherFactory.createTextCheckMatcher("top"));
    public static final TokenId UNDER = new TokenId("under", UNDER_INT, null, MatcherFactory.createTextCheckMatcher("_"));
    public static final TokenId UPPER_CASE_LETTER = new TokenId("upper-case-letter", UPPER_CASE_LETTER_INT);
    public static final TokenId WHITESPACE = new TokenId("whitespace", WHITESPACE_INT);

    PepaLanguage() {
    }

    public Lexer createLexer() {
        return new PepaLexer();
    }

}
