package eduni.inf.nbmodules.pepa.loader;

import eduni.inf.nbmodules.pepa.actions.DeriveMenuAction;
import eduni.inf.nbmodules.pepa.actions.PepaStateFinderAction;
import eduni.inf.nbmodules.pepa.actions.ReflectAction;
import eduni.inf.nbmodules.pepa.actions.SingleStepDebuggerAction;
import eduni.inf.nbmodules.pepa.actions.SolveMenuAction;
import eduni.inf.nbmodules.pepa.actions.SyntaxCheckAction;
import eduni.inf.nbmodules.pepa.actions.ThroughputDialogAction;
import eduni.inf.nbmodules.pepa.actions.VisualSingleStepDebuggerAction;
import java.io.IOException;
import org.openide.actions.CopyAction;
import org.openide.actions.CutAction;
import org.openide.actions.DeleteAction;
import org.openide.actions.OpenAction;
import org.openide.actions.PropertiesAction;
import org.openide.actions.RenameAction;

import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.FileEntry;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.loaders.FileEntry.Numb;

import org.openide.loaders.MultiDataObject.Entry;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;




/** Recognizes single files in the Repository as being PEPA.
 *
 * @author N.V.Haenel
 */
public class PepaDataLoader extends MultiFileLoader {
    
    private static final long serialVersionUID = 1L;
    
    
    static final String PEPA_EXTENSION = "pepa";
    
    static final String PEPA_SEQ_EXTENSION = "seq";
    
    static final String PEPA_STATES_EXTENSION = "states";
    
    static final String PEPA_STEADYSTATE_EXTENSION = "steadystate";
    
    static final String PEPA_SORSTATE_EXTENSION = "sorstate";
        
    static final String PEPA_XML_RESULTS_EXTENSION = "xml";
    
    static final String PEPA_STATE_FINDER= "psf";
    
    static final String PEPA_THROUGHPUT= "troughput";
    
    
    
    public PepaDataLoader() {
        this("eduni.inf.nbmodules.pepa.loaders.PepaDataObject");   
        
    }
    
    // Can be useful for subclasses:
    protected PepaDataLoader(String recognizedObjectClass) {
        super(recognizedObjectClass);
    }
    
    protected String defaultDisplayName() {
        return NbBundle.getMessage(PepaDataLoader.class, "PepaDataLoader");
    }
    
    protected void initialize() {
        super.initialize();
    }
    
    protected SystemAction[] defaultActions() {
        return new SystemAction[] {            
            SystemAction.get(OpenAction.class),
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            null,
            SystemAction.get(SyntaxCheckAction.class),
            SystemAction.get(DeriveMenuAction.class),
            SystemAction.get(SolveMenuAction.class),
            SystemAction.get(ReflectAction.class),                                                
            null,
            SystemAction.get(PepaStateFinderAction.class),                                                
            SystemAction.get(ThroughputDialogAction.class),                                                
            SystemAction.get(SingleStepDebuggerAction.class),
	    //SystemAction.get(VisualSingleStepDebuggerAction.class),                                                                         
            null,
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            null,
            SystemAction.get(PropertiesAction.class),            
        };
    }
    
    protected MultiDataObject createMultiObject(FileObject primaryFile) throws DataObjectExistsException, IOException {
        return new PepaDataObject(primaryFile, this);
    }         
    
    protected Entry createPrimaryEntry(MultiDataObject multiDataObject, FileObject primaryFileObject) {
        primaryFileObject.setImportant(true);
        return new FileEntry(multiDataObject, primaryFileObject);

    }    
    
    protected Entry createSecondaryEntry(MultiDataObject multiDataObject, FileObject secondaryFileObject) {
        secondaryFileObject.setImportant(true);
        return new Numb(multiDataObject, secondaryFileObject);
    }    
    
    protected FileObject findPrimaryFile(FileObject fileObject) {
        // never recognize folders.
        if (fileObject.isFolder()) return null;
        if(fileObject.getExt().equals(PEPA_EXTENSION))
            return fileObject;
        // if we find a errors file, return the associated primary file
        else if (fileObject.getExt().equals(PEPA_SEQ_EXTENSION) ||
                fileObject.getExt().equals(PEPA_STATES_EXTENSION) ||
                fileObject.getExt().equals(PEPA_XML_RESULTS_EXTENSION) ||
                fileObject.getExt().equals(PEPA_STEADYSTATE_EXTENSION) ||
                fileObject.getExt().equals(PEPA_STATE_FINDER) ||
                
                fileObject.getExt().equals(PEPA_SORSTATE_EXTENSION)    )
            return FileUtil.findBrother(fileObject, PEPA_EXTENSION);
        return null;
    }
    
    
    
}
