package eduni.inf.nbmodules.pepa.loader;

import eduni.inf.nbmodules.pepa.cookies.ReflectionCookie;
import eduni.inf.nbmodules.pepa.editor.PepaEditorSupport;
import eduni.inf.nbmodules.pepa.settings.PepaSettings;
import eduni.inf.pepa.pepa.Peparoni;
import eduni.inf.pepa.pepamodel.PepaModel;
import eduni.inf.pepa.workbench.Reporter;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.Children;
import org.openide.nodes.Children.Array;

import org.openide.nodes.CookieSet;
import org.openide.nodes.Node;
import org.openide.nodes.Node.Cookie;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.windows.TopComponent;

/** Represents a PEPA Object in the filesystem.
 *
 * @author N.V.Haenel
 */
public class PepaDataObject extends MultiDataObject implements PropertyChangeListener{
    
    private Peparoni driver;
    
    private List dialogs;
    public PepaDataObject(FileObject pf, PepaDataLoader loader) throws DataObjectExistsException {
        super(pf, loader);
        init();
    }
    
    private void init() {
        CookieSet cookies = getCookieSet();
        cookies.add(new PepaEditorSupport(this));
        this.addPropertyChangeListener(this);
        
        Peparoni.initialise();
        driver = new Peparoni();
        dialogs = new ArrayList();
        PepaSettings s = (PepaSettings)SystemOption.findObject(PepaSettings.class, true);        	    
            
        Reporter.get().setVerbose(s.isVerbose());        
    }
    
    public Peparoni getPeparoni(){
        return this.driver;
    }
    
    public PepaModel getPepaModel(){
        PepaEditorSupport pes = (PepaEditorSupport)this.getCookie(PepaEditorSupport.class);
        return pes.getPepaModel();
    }
    
    public void addDialog(TopComponent c){
        dialogs.add(c);
    }
    
    public void clearAllDialogs(){
        for(Iterator i = dialogs.iterator();i.hasNext();){
            ((TopComponent)i.next()).close();
        }
        dialogs.clear();
    }
    
    /** Reset the list of children belonging to the <code>PepaNetDataNode</code>
     *  of this data object. Also check for XML files.
     *
     *
     */
    synchronized private  Children synchronizeChildren(){
        Node[] nodeArray = new Node[this.secondaryEntries().size()];
        Entry[] entryArray = new Entry[this.secondaryEntries().size()];
        entryArray = (Entry[])secondaryEntries().toArray(entryArray);
        
        for (int i = 0;i<entryArray.length ;i++){
            if(entryArray[i].getFile().hasExt(PepaDataLoader.PEPA_XML_RESULTS_EXTENSION))
                nodeArray[i] = new PepaSecondaryNode(entryArray[i],  "text/xml");
            
            else
                nodeArray[i] = new PepaSecondaryNode(entryArray[i], "text");
        }
        
        Children children = new Array();
        children.add(nodeArray);
        return children;
    }
    
    /** If a target exists for reflection, add a ReflectionCookie.
     */
    public boolean reflectionTarget(){
        try{
            String file = this.getPrimaryFile().getPath();
            String xmi = file.replaceFirst(".pepa", ".xmi");
            String zuml = file.replaceFirst(".pepa", ".zuml");
            
            FileObject xmiFO;
            FileObject zumlFO;
            
            List targets = new ArrayList();
            
            if ((xmiFO = this.getPrimaryFile().getFileSystem().findResource(xmi)) != null){
                targets.add(xmiFO);
            }
            if ((zumlFO = this.getPrimaryFile().getFileSystem().findResource(zuml)) != null)
                targets.add(zumlFO);
            if(!targets.isEmpty()){
                this.addCookie(new ReflectionCookie(targets, this));
                return true;
            }
        }catch(FileStateInvalidException fsie){
            Reporter.get().reportWarning(fsie.getMessage());
        }
        return false;
        
    }
    
    private FileObject getSecondaryFile(String ext){
        for (Iterator i = this.secondaryEntries().iterator();i.hasNext();){
            Entry current = (Entry)i.next();
            if(current.getFile().hasExt(ext)){
                return current.getFile();
            }
            
        }
        return null;
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        // If you add context help, change to:
        // return new HelpCtx(MyDataObject.class);
    }
    
    protected Node createNodeDelegate() {
        return new PepaDataNode(this,this.synchronizeChildren());
    }
    
    
    
    
    public final void addSaveCookie(SaveCookie save) {
        getCookieSet().add(save);
    }
    
    public final void removeSaveCookie(SaveCookie save) {
        getCookieSet().remove(save);
    }
    
    
    public final void addCookie(Cookie cookie){
        this.getCookieSet().add(cookie);
    }
    
    public final void removeCookie(Cookie cookie){
        this.getCookieSet().remove(cookie);
        
    }
    
    
    /** When the <code>files</code> property changes, for example if a file is generated or deleted
     *  we update the children.
     *
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if(evt.getPropertyName()=="files"){
            ((PepaDataNode)this.getNodeDelegate()).updateChildren(this.synchronizeChildren());
        }
        
    }
    
    
}
