package eduni.inf.nbmodules.pepa.loader;

import eduni.inf.nbmodules.pepa.editor.PepaSecondaryEditor;
import eduni.inf.nbmodules.pepa.net.editor.PepaNetSecondaryEditor;
import java.io.IOException;
import javax.swing.Action;
import org.openide.actions.DeleteAction;
import org.openide.actions.OpenAction;
import org.openide.loaders.MultiDataObject.Entry;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;


/** Represents a Secondary entry for a <code>PepaNetDataObject</code>.
 *
 * @see PepaNetDataObject
 * 
 * @author N.V.Haenel
 */
public class PepaSecondaryNode extends AbstractNode {
    
    private Entry entry;
    
    public PepaSecondaryNode(Entry entry, String mimeType) {
        super(Children.LEAF);
        this.entry = entry;        
        this.setName(entry.getFile().getExt()); 
        this.setDisplayName(entry.getFile().getNameExt());        
        this.getCookieSet().add(new PepaSecondaryEditor((PepaDataObject)entry.getDataObject(), this, mimeType));                                                
        //setIconBase("eduni/inf/nbmodules/pepa/resources/PepaDataIcon");
    }
    
    public Entry getEntry(){
        return entry;
    }
            
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    public Action getPreferredAction(){
        return (Action)SystemAction.get(OpenAction.class);
    }
    
    
    public Action[] getActions(boolean b){
                return new Action[] {
            (Action)SystemAction.get(OpenAction.class),
            null,
            (Action)SystemAction.get(DeleteAction.class),                                
        };

    }
    
    public boolean canDestroy() {
        return true;
    }
    
    public void destroy() throws IOException {
        this.entry.getFile().delete();
    }             
}
