package eduni.inf.nbmodules.pepa.settings;

import eduni.inf.pepa.pepa.Peparoni;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Settings for the Lnbcg Solver.
 *
 * @author N.V.Haenel
 */
public class LnbcgSettings extends SystemOption {
    
    private static final long serialVersionUID = 1L;
            
    // iteration tolerance range 1-4
    private static final String PROP_ITOL = "itol";
    // maximum iterations
    private static final String PROP_MAXITERS = "maxiters";
    // tolerance of algorithm
    private static final String PROP_TOLERANCE = "tolerance";
    // generalised minimal residual variant
    private static final String PROP_GMRES = "gmres";
    
    
    // No constructor please!
    
    protected void initialize() {
        super.initialize();
    }
    
    public String displayName() {
        return NbBundle.getMessage(LnbcgSettings.class, "LNBCG_settings");
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        
    }
    
    /** Default instance of this system option, for the convenience of associated classes. */
    public static LnbcgSettings getDefault() {
        return (LnbcgSettings)findObject(LnbcgSettings.class, true);
    }
    
    public int getItol(){
        if ((Integer)getProperty(PROP_ITOL)==null){
            this.putProperty(PROP_ITOL, new Integer(1));
        }
        return ((Integer)getProperty(PROP_ITOL)).intValue();
    }
    
    public void setItol(int i){
        this.putProperty(PROP_ITOL, new Integer(i));
    }
    
    public int getMaxiters(){
        if ((Integer)getProperty(PROP_MAXITERS)==null){
            this.putProperty(PROP_MAXITERS, new Integer(100));
        }
        return ((Integer)getProperty(PROP_MAXITERS)).intValue();
    }
    
    public void setMaxiters(int i){
        this.putProperty(PROP_MAXITERS, new Integer(i));
    }
    
    public double getTolerance(){
        if ((Double )getProperty(PROP_TOLERANCE)==null){
            this.putProperty(PROP_TOLERANCE, new Double(Peparoni.toldef));
        }
        return ((Double)getProperty(PROP_TOLERANCE)).doubleValue();
    }
    
    public void  setTolerance(double d){
        this.putProperty(PROP_TOLERANCE, new Double(d));
    }
    
    public boolean isGmres(){
        if((Boolean)getProperty(PROP_GMRES) ==null){
            this.putProperty(PROP_GMRES, new Boolean(true));
        }
        return ((Boolean)getProperty(PROP_GMRES)).booleanValue();
        
    }
    
    
    public void setGmres(boolean b){
        putProperty(PROP_GMRES, b ? Boolean.TRUE : Boolean.FALSE);
        
    }
    
    
}
