package eduni.inf.nbmodules.pepa.settings;

import java.awt.Image;
import java.beans.*;

import org.openide.ErrorManager;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/** Description of {@link PepaSettings}.
 *
 * @author N.V.Haenel
 */
public class PepaSettingsBeanInfo extends SimpleBeanInfo {
    
    public PropertyDescriptor[] getPropertyDescriptors() {
        try {
            
            PropertyDescriptor solver = new PropertyDescriptor("solver", PepaSettings.class);
            solver.setDisplayName(NbBundle.getMessage(PepaSettingsBeanInfo.class, "PROP_solver"));
            solver.setShortDescription(NbBundle.getMessage(PepaSettingsBeanInfo.class, "HINT_solver"));
            solver.setPropertyEditorClass(SolverEditor.class);
            
            PropertyDescriptor output = new PropertyDescriptor("output", PepaSettings.class);
            output.setDisplayName(NbBundle.getMessage(PepaSettingsBeanInfo.class, "PROP_output"));
            output.setShortDescription(NbBundle.getMessage(PepaSettingsBeanInfo.class, "HINT_output"));
            output.setPropertyEditorClass(OutputEditor.class);
            
            PropertyDescriptor backgrnd = new PropertyDescriptor("backgrnd", PepaSettings.class);
            backgrnd.setDisplayName(NbBundle.getMessage(PepaSettingsBeanInfo.class, "PROP_backgrnd"));
            backgrnd.setShortDescription(NbBundle.getMessage(PepaSettingsBeanInfo.class, "HINT_backgrnd"));
            
            PropertyDescriptor bgrndtime = new PropertyDescriptor("bgrndtime", PepaSettings.class);
            bgrndtime.setDisplayName(NbBundle.getMessage(PepaSettingsBeanInfo.class, "PROP_bgrndtime"));
            bgrndtime.setShortDescription(NbBundle.getMessage(PepaSettingsBeanInfo.class, "HINT_bgrndtime"));
            
            PropertyDescriptor verbose = new PropertyDescriptor("verbose", PepaSettings.class);
            verbose.setDisplayName(NbBundle.getMessage(PepaSettingsBeanInfo.class, "PROP_verbose"));
            verbose.setShortDescription(NbBundle.getMessage(PepaSettingsBeanInfo.class, "HINT_verbose"));
            
            
            
            
            return new PropertyDescriptor[] {solver,output,backgrnd,bgrndtime,verbose};
        } catch (IntrospectionException ie) {
            ErrorManager.getDefault().notify(ie);
            return null;
        }
    }
    
  
    public static class SolverEditor extends PropertyEditorSupport{
        
        private static final String[] tags = {"LNBCG Solver","SOR Solver"};        
         
        private static final int[] levels = {1,2};
        
        public String[] getTags() {  return tags;    }
        
        public String getAsText() {
            int l = ((Integer) getValue()).intValue();
            for (int i = 0; i < levels.length; i++) {
                if (l == levels[i]) {
                    return tags[i];
                }
            }
            return "???"; // NOI18N
        }
        
        public void setAsText(String text) throws IllegalArgumentException {
            for (int i = 0; i < tags.length; i++) {
                if (tags[i].equals(text)) {
                    setValue(new Integer(levels[i]));
                    return;
                }
            }
            throw new IllegalArgumentException();
        }
    }
    
    public static class OutputEditor extends PropertyEditorSupport{
        
        private static final String[] tags = {"Silent","Compressed", "Uncompressed"};        
         
        private static final int[] levels = {1,2,3};
        
        public String[] getTags() {  return tags;    }
        
        public String getAsText() {
            int l = ((Integer) getValue()).intValue();
            for (int i = 0; i < levels.length; i++) {
                if (l == levels[i]) {
                    return tags[i];
                }
            }
            return "???"; // NOI18N
        }
        
        public void setAsText(String text) throws IllegalArgumentException {
            for (int i = 0; i < tags.length; i++) {
                if (tags[i].equals(text)) {
                    setValue(new Integer(levels[i]));
                    return;
                }
            }
            throw new IllegalArgumentException();
        }
    }
}
