package eduni.inf.nbmodules.pepa.settings;

import eduni.inf.pepa.pepa.Peparoni;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/** Settings for the Sor Solver.
 *
 * @author N.V.Haenel
 */
public class SorSettings extends SystemOption {
    
    private static final long serialVersionUID = 1L;
            
    // iteration tolerance range 1-4
    private static final String PROP_OMEGA = "omega";
    // maximum iterations
    private static final String PROP_MAXITERS = "maxiters";
    // tolerance of algorithm
    private static final String PROP_TOLERANCE = "tolerance";
    // generalised minimal residual variant
    private static final String PROP_NORM = "norm";
    
    
    // No constructor please!
    
    protected void initialize() {
        super.initialize();
    }
    
    public String displayName() {
        return NbBundle.getMessage(SorSettings.class, "SOR_Settings");
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        
    }
    
    /** Default instance of this system option, for the convenience of associated classes. */
    public static SorSettings getDefault() {
        return (SorSettings)findObject(SorSettings.class, true);
    }
    
    public double getOmega(){
        if ((Double)getProperty(PROP_OMEGA)==null){
            this.putProperty(PROP_OMEGA, new Double(1.0));
        }
        return ((Double)getProperty(PROP_OMEGA)).doubleValue();
    }
    
    public void setOmega(double i){
        this.putProperty(PROP_OMEGA, new Double(i));
    }
    
    public int getMaxiters(){
        if ((Integer)getProperty(PROP_MAXITERS)==null){
            this.putProperty(PROP_MAXITERS, new Integer(100));
        }
        return ((Integer)getProperty(PROP_MAXITERS)).intValue();
    }
    
    public void setMaxiters(int i){
        this.putProperty(PROP_MAXITERS, new Integer(i));
    }
    
    public double getTolerance(){
        if ((Double )getProperty(PROP_TOLERANCE)==null){
            this.putProperty(PROP_TOLERANCE, new Double(Peparoni.toldef));
        }
        return ((Double)getProperty(PROP_TOLERANCE)).doubleValue();
    }
    
    public void  setTolerance(double d){
        this.putProperty(PROP_TOLERANCE, new Double(d));
    }
    
    public int getNorm(){
        if ((Integer)getProperty(PROP_NORM)==null){
            this.putProperty(PROP_NORM, new Integer(1));
        }
        return ((Integer)getProperty(PROP_NORM)).intValue();
    }
    
    public void setNorm(int i){
        this.putProperty(PROP_NORM, new Integer(i));
    }
    
    
    
    
    
}
