package eduni.inf.nbmodules.pepa.settings;

import java.awt.Image;
import java.beans.*;

import org.openide.ErrorManager;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/** Description of {@link SorSettings}.
 *
 * @author N.V.Haenel
 */
public class SorSettingsBeanInfo extends SimpleBeanInfo {
    
    public PropertyDescriptor[] getPropertyDescriptors() {
        try {
            
            PropertyDescriptor omega = new PropertyDescriptor("omega", SorSettings.class);
            omega.setDisplayName(NbBundle.getMessage(SorSettingsBeanInfo.class, "PROP_omega"));
            omega.setShortDescription(NbBundle.getMessage(SorSettingsBeanInfo.class, "HINT_omega"));
            
            PropertyDescriptor maxiters = new PropertyDescriptor("maxiters", SorSettings.class);
            maxiters.setDisplayName(NbBundle.getMessage(SorSettingsBeanInfo.class, "PROP_maxiters"));
            maxiters.setShortDescription(NbBundle.getMessage(SorSettingsBeanInfo.class, "HINT_maxiters"));
            //maxiters.setPropertyEditorClass(IntegerEditor.class);
            
            PropertyDescriptor tolerance = new PropertyDescriptor("tolerance", SorSettings.class);
            tolerance.setDisplayName(NbBundle.getMessage(SorSettingsBeanInfo.class, "PROP_tolerance"));
            tolerance.setShortDescription(NbBundle.getMessage(SorSettingsBeanInfo.class, "HINT_tolerance"));
            
            PropertyDescriptor norm = new PropertyDescriptor("norm", SorSettings.class);
            norm.setDisplayName(NbBundle.getMessage(SorSettingsBeanInfo.class, "PROP_norm"));
            norm.setShortDescription(NbBundle.getMessage(SorSettingsBeanInfo.class, "HINT_norm"));
            norm.setPropertyEditorClass(NormEditor.class);
            
            
            
            
            
            return new PropertyDescriptor[] {omega,maxiters,tolerance,norm};
        } catch (IntrospectionException ie) {
            ErrorManager.getDefault().notify(ie);
            return null;
        }
    }
            
    public static class NormEditor extends PropertyEditorSupport {
        
        private static final String[] tags = {"Infinity Norm","Linear Norm"};
        
        private static final int[] levels = {1,2};
        
        public String[] getTags() {  return tags;    }
                
        public String getAsText () {
            int l = ((Integer) getValue ()).intValue ();
            for (int i = 0; i < levels.length; i++) {
                if (l == levels[i]) {
                    return tags[i];
                }
            }
            return "???"; // NOI18N
        }

        public void setAsText (String text) throws IllegalArgumentException {
            for (int i = 0; i < tags.length; i++) {
                if (tags[i].equals (text)) {
                    setValue (new Integer (levels[i]));
                    return;
                }
            }
            throw new IllegalArgumentException ();
        }                
    }    
}
