package eduni.inf.nbmodules.xmi.actions;

import eduni.inf.nbmodules.lysa.settings.LysaSettings;
import eduni.inf.nbmodules.xmi.loaders.XmiDataObject;
import eduni.inf.nbmodules.xmi.loaders.ZumlDataObject;
import lysaExtractor.Lysa_Output;
import java.io.File;
import java.io.IOException;
import org.openide.ErrorManager;
import org.openide.cookies.EditorCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.options.SystemOption;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;




import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;
import util.Trace;



/** Action to extract LYSA model from an XMI file.
 * @author N.V.Haenel
 */
public class ExtractLysaAction extends CookieAction {
    
    protected Class[] cookieClasses() {
        return new Class[] {ZumlDataObject.class, XmiDataObject.class};
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
    }
    
    
    protected void performAction(Node[] nodes) {
        
        DataObject dataObject = (DataObject)nodes[0].getCookie(DataObject.class);
        
        //if(dataObject.getClass().getName().equals("eduni.inf.nbmodules.xmi.loaders.XmiDataObject")){
        this.performExtraction(dataObject);
        
        
    }
    
    protected void performExtraction(DataObject dataObject){
        try{
            // get the io tab for LySa
            InputOutput io  = IOProvider.getDefault().getIO("XMI", false);
            io.setFocusTaken(true);
            io.getOut().reset();
            io.select();
            
            // redirect the output from the LySa extractor
            Trace.setExternal(true);
            Trace.setPrintWriter(io.getOut());
            
            // get the file
            File file = FileUtil.toFile(dataObject.getPrimaryFile());
            //invoke extractor
            this.invokeExtractor(file.toString());
            // show file in explorer
            dataObject.getPrimaryFile().getFileSystem().refresh(true);
            // and open in an editor
            FileObject fo = FileUtil.fromFile(Lysa_Output.lysaFile)[0];
            if(fo != null){
                DataObject d = DataObject.find(fo);
                EditorCookie ec = (EditorCookie)d.getCookie(EditorCookie.class);
                ec.open();
                ec.openDocument();
            }
            
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);
        }
    }
    
    
    
    protected void invokeExtractor(String filename){
        LysaSettings ls = (LysaSettings)SystemOption.findObject(LysaSettings.class, true);
        File lysa = ls.getLysaheapChecked();
        boolean t = ls.getTraceoutput();
        
        if(t)        
            Lysa_Output.main(new String[]{ "-t",filename});
        else
            Lysa_Output.main(new String[]{filename});
        
    }
    
    
    public String getName() {
        return NbBundle.getMessage(ExtractLysaAction.class, "ExtractLysa_Action");
    }
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/xmi/actions/icons/ExtractLysaActionIcon.gif";
        
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    protected boolean asynchronous() {
        return true;
    }
    
}
