package eduni.inf.nbmodules.xmi.actions;

import eduni.inf.nbmodules.xmi.loaders.XmiDataObject;
import eduni.inf.nbmodules.xmi.loaders.ZumlDataObject;
import eduni.inf.pepa.net.extractor.PEPANet_Extractor;
import eduni.inf.pepa.pepa.Peparoni;
import eduni.inf.pepa.reflector.Reflector;
import eduni.inf.pepa.workbench.Reporter;

import java.io.File;
import java.io.IOException;
import org.openide.ErrorManager;
import org.openide.cookies.EditorCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;


import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;



/** Action to extract a PEPA model from an XMI or ZUML file.
 *
 *
 * @author N.V.Haenel
 */
public class ExtractPepaNetAction extends CookieAction {
    
    
    protected Class[] cookieClasses() {
        return new Class[] {ZumlDataObject.class, XmiDataObject.class};
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
    }
    
    
    protected void performAction(Node[] nodes) {
        DataObject dataObject = (DataObject)nodes[0].getCookie(DataObject.class);
        this.performExtraction(dataObject);                
    }
    
    protected void performExtraction(DataObject dataObject){
        try{
            // get the io tab
            InputOutput io  = IOProvider.getDefault().getIO("XMI", false);
            io.setFocusTaken(true);
            io.getOut().reset();
            io.select();
            // redirect output from the PEPA workbench
            Reporter.get().setExternalMode(true);
            Reporter.get().setPrintWriter(io.getOut());
            
            PEPANet_Extractor e = PEPANet_Extractor.getPEPANet_Extractor();
            
            File umlFile = FileUtil.toFile(dataObject.getPrimaryFile());
            
            File pepanet = this.extract(e, umlFile);
                                                                    
            // refresh file system
            dataObject.getPrimaryFile().getFileSystem().refresh(true);
            // and open in editor
            FileObject fo = FileUtil.fromFile(pepanet)[0];
            DataObject d = DataObject.find(fo);
            EditorCookie ec = (EditorCookie)d.getCookie(EditorCookie.class);
            ec.open();
            ec.openDocument();
             
            
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);
            
        }
    }
    
    protected File extract(PEPANet_Extractor e, File umlFile){
        File returnFile = e.extractPepaNet(umlFile);
        return returnFile;
    }

    
    public String getName() {
        return NbBundle.getMessage(ExtractPepaAction.class, "ExtractPepaNetAction");
    }
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/xmi/actions/icons/ExtractPepaNetAction.gif";
    }
    
    protected boolean asynchronous(){
        return true;
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
}
