package eduni.inf.nbmodules.xmi.actions;

import eduni.inf.nbmodules.xmi.loaders.ZumlDataObject;
import java.io.*;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import org.openide.ErrorManager;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;


/** Action to unzip .zuml archive files.
 *
 * @author N.V.Haenel
 */
public class UnzipZumlAction extends CookieAction {
    
    protected Class[] cookieClasses() {
        return new Class[] {ZumlDataObject.class};
    }
    
    protected int mode() {
        return MODE_EXACTLY_ONE;
    }
    
    protected void performAction(Node[] nodes) {
        ZumlDataObject zdo = (ZumlDataObject)nodes[0].getCookie(ZumlDataObject.class);
        
        File f = FileUtil.toFile(zdo.getPrimaryFile());
        
        try{
            ZipFile zip = new ZipFile(f);
            
            for(Enumeration e = zip.entries();e.hasMoreElements();){
                ZipEntry next = (ZipEntry)e.nextElement();
                File file = new File(f.getParent()+f.separator+next.getName());
                InputStream is = zip.getInputStream(next);
                this.zipEntryToFile(file, is);
            }
            
            zip.close();
            
            InputOutput io  = IOProvider.getDefault().getIO("XMI", false);
            io.setFocusTaken(true);
            io.getOut().reset();
            io.select();
            io.getOut().write(zdo.getPrimaryFile().getName() +" has been unzipped");
            zdo.getPrimaryFile().getParent().refresh();
            
            
        }catch(ZipException ze){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ze);
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);
        }
        
        
    }
    /** Write the zip files entry to a new file in that directory */
    public void zipEntryToFile(File file, InputStream is){
        try{
            BufferedInputStream bis = new BufferedInputStream(is);
            BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file));
            
            byte[] data = new byte[10240];
            while (bis.available() != 0) {
                int read = bis.read(data, 0, data.length);
                if (read == -1)
                    break;
                bos.write(data, 0, read);
            }
            bis.close();
            bos.flush();
            bos.close();
        }catch(IOException ioe){
            ErrorManager.getDefault().notify(ErrorManager.EXCEPTION, ioe);
        }
    }
    
    public String getName() {
        return NbBundle.getMessage(UnzipZumlAction.class, "UnzipZumlAction");
    }
    
    protected String iconResource() {
        return "eduni/inf/nbmodules/xmi/actions/icons/UnzipZumlActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx(UnzipZumlAction.class);
    }
    
    protected boolean asynchronous() {
        return true;
    }
    
    /** Perform special enablement check in addition to the normal one.
     * protected boolean enable(Node[] nodes) {
     * if (!super.enable(nodes)) return false;
     * if (...) ...;
     * }
     */
    
    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
     * protected void initialize() {
     * super.initialize();
     * putProperty(Action.SHORT_DESCRIPTION, NbBundle.getMessage(UnzipZumlAction.class, "HINT_Action"));
     * }
     */
    
}
