package eduni.inf.nbmodules.xmi.loaders;

import eduni.inf.nbmodules.xmi.actions.ExtractLysaDiagram;
import eduni.inf.nbmodules.xmi.actions.ExtractPepaDiagram;
import eduni.inf.nbmodules.xmi.actions.ExtractPepaNetDiagram;
import eduni.inf.nbmodules.xmi.cookies.ActivityCookie;
import eduni.inf.nbmodules.xmi.cookies.CollaborationCookie;
import eduni.inf.nbmodules.xmi.cookies.SequenceCookie;
import javax.swing.Action;
import org.openide.actions.ToolsAction;
import org.openide.loaders.DataObject;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Node.Cookie;

import org.openide.nodes.Children;
import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;


/** Represents a UML Diagram.
 *
 * @author N.V.Haenel
 */
public class UmlDiagramNode extends AbstractNode implements Cookie{
    
    private DataObject d;
    private String name;
    private String type;
    
    private String collaboration = "CollaborationDiagram";
    private String sequence = "SequenceDiagram";
    private String activity = "ActivityDiagram";
    
    
    
    public UmlDiagramNode(DataObject d, String name, String type) {
        super(Children.LEAF);
        this.setDisplayName(name);
        String icon = "";
        if(type.equals(collaboration)){
            this.getCookieSet().add(new CollaborationCookie());
            icon = "CollaborationDiagramIcon";
        }
        else if (type.equals(sequence)){
            this.getCookieSet().add(new SequenceCookie());
            
            icon = "SequenceDiagramIcon";
        }
        else if (type.equals(activity)){
            this.getCookieSet().add(new ActivityCookie());
            
            icon = "ActivityDiagramIcon";
        }
        
        this.getCookieSet().add(this);
        this.type = type;
        
        this.setIconBase("eduni/inf/nbmodules/xmi/resources/"+icon);
        this.d = d;
        this.name = name;
    }
    
    public Action[] getActions(boolean b) {
        if(type.equals(collaboration))
            return new Action[] {
                (Action)SystemAction.get(ExtractPepaDiagram.class),
            };
        else if(type.equals(sequence))
            return new Action[] {
                (Action)SystemAction.get(ExtractLysaDiagram.class),
            };
        else /*if(type.equals(activity))*/
            return new Action[] {
                (Action)SystemAction.get(ExtractPepaNetDiagram.class),
            };
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    public Action getPreferredAction(){
        if(type.equals(collaboration))
            return (Action)SystemAction.get(ExtractPepaDiagram.class);
        else if(type.equals(sequence))
            return (Action)SystemAction.get(ExtractLysaDiagram.class);
        else /*if(type.equals(activity))*/
            return (Action)SystemAction.get(ExtractPepaNetDiagram.class);
        
        
    }
    
    public DataObject getDataObject(){
        return d;
    }
    
    public String getDiagramName(){
        return name;
    }
    
}