package eduni.inf.nbmodules.xmi.loaders;

import eduni.inf.nbmodules.xmi.actions.ExtractLysaAction;
import eduni.inf.nbmodules.xmi.actions.ExtractPepaAction;
import eduni.inf.nbmodules.xmi.actions.ExtractPepaNetAction;
import eduni.inf.nbmodules.xmi.actions.GenerateNarrationAction;
import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.ExtensionList;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.UniFileLoader;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;




/** Recognizes single files in the Repository as being XMI.
 *
 * @author N.V.Haenel
 */
public class XmiDataLoader extends UniFileLoader {
    
    private static final long serialVersionUID = 1L;
    
    static final String MIME_TYPE = "text/xmi+xml";
    
    static final String XMI_EXTENSION = "xmi";
    
    public XmiDataLoader() {
        super("eduni.inf.nbmodules.xmi.loaders.XmiDataObject");
        
    }
    
    protected String defaultDisplayName() {
        return NbBundle.getMessage(XmiDataLoader.class, "XmiDataLoader");
    }
    
    protected void initialize() {
        super.initialize();
        
        ExtensionList extensions = new ExtensionList();
        extensions.addExtension(XMI_EXTENSION);
        extensions.addMimeType(MIME_TYPE);
        setExtensions(extensions);
    }
    
    protected SystemAction[] defaultActions() {
        return new SystemAction[] {
            SystemAction.get(OpenAction.class),
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            SystemAction.get(PasteAction.class),
            null,
            SystemAction.get(ExtractLysaAction.class),
            SystemAction.get(ExtractPepaAction.class),            
            SystemAction.get(ExtractPepaNetAction.class),
            null,
            SystemAction.get(GenerateNarrationAction.class),            
            null,
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            null,
            SystemAction.get(PropertiesAction.class),
        };
    }
    
    protected MultiDataObject createMultiObject(FileObject primaryFile) throws DataObjectExistsException, IOException {
        return new XmiDataObject(primaryFile, this);
    }
    
    
}
