package eduni.inf.nbmodules.xmi.loaders;

import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;

import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.CookieSet;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;

import eduni.inf.nbmodules.xmi.editor.*;
import eduni.inf.pepa.extractor.PEPA_Extractor2;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.openide.filesystems.FileUtil;
import org.openide.nodes.Children;
import org.openide.nodes.Children.Array;



/** Represents a XMI object in the Repository.
 *
 * @author N.V.Haenel
 */
public class XmiDataObject extends MultiDataObject  {
    
    
    
    
    public XmiDataObject(FileObject pf, XmiDataLoader loader) throws DataObjectExistsException {
        super(pf, loader);
        init();
    }
    
    private void init() {
        CookieSet cookies = getCookieSet();
        CookieSet set = getCookieSet();
        set.add(new XmiEditorSupport(this));
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    protected Node createNodeDelegate() {
        return new XmiDataNode(this, this.getChildren());
    }
    
    public final void addSaveCookie(SaveCookie save) {
        getCookieSet().add(save);
    }
    
    public final void removeSaveCookie(SaveCookie save) {
        getCookieSet().remove(save);
    }
    
    
    
    protected Children getChildren(){
        Children children = new Array();
        ArrayList nodes = new ArrayList();
        
               PEPA_Extractor2 p = new PEPA_Extractor2();
        
        if(p.parse(FileUtil.toFile(this.getPrimaryFile()))){
            List c= p.getAllDiagramsStrings(Arrays.asList(new String[]{"CollaborationDiagram"}));
            for(Iterator i = c.iterator();i.hasNext();){
                nodes.add(new UmlDiagramNode(this, (String)i.next() , "CollaborationDiagram"));
            }
            List d= p.getAllDiagramsStrings(Arrays.asList(new String[]{"SequenceDiagram"}));
            for(Iterator i = d.iterator();i.hasNext();){
                nodes.add(new UmlDiagramNode(this, (String)i.next(), "SequenceDiagram"));
            }
            List e= p.getAllDiagramsStrings(Arrays.asList(new String[]{"ActivityDiagram"}));
            for(Iterator i = e.iterator();i.hasNext();){
                nodes.add(new UmlDiagramNode(this, (String)i.next() , "ActivityDiagram"));
            }
            
        }
        
        
        Node[] nodeArray = new Node[nodes.size()];
        children.add((Node[])nodes.toArray(nodeArray));
        return children;
    }
    
    
}
