package eduni.inf.nbmodules.xmi.loaders;

import eduni.inf.nbmodules.xmi.actions.*;

import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.ExtensionList;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.UniFileLoader;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;




/** Recognizes single files in the Repository as being ZUML(Poseidon).
 *  archives.
 *
 * @author N.V.Haenel
 */
public class ZumlDataLoader extends UniFileLoader {
    
    private static final long serialVersionUID = 1L;
        
    static final String ZUML_EXTENSION = "zuml";
    
    public ZumlDataLoader() {        
        super("eduni.inf.nbmodules.xmi.loaders.ZumlDataObject");
            
    }
        
    protected String defaultDisplayName() {
        return NbBundle.getMessage(XmiDataLoader.class, "ZumlDataLoader");
    }
    
    protected void initialize() {
        super.initialize();
        
        ExtensionList extensions = new ExtensionList();
        extensions.addExtension(ZUML_EXTENSION);  
        setExtensions(extensions);        
    }
    
    protected SystemAction[] defaultActions() {
        return new SystemAction[] {
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            SystemAction.get(PasteAction.class),
            null,
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            null,
            SystemAction.get(UnzipZumlAction.class),            
            SystemAction.get(RefreshZumlAction.class),            
            null,            
            SystemAction.get(ExtractLysaAction.class),            
            SystemAction.get(ExtractPepaAction.class),                                    
            SystemAction.get(ExtractPepaNetAction.class),                                                
            null,
            SystemAction.get(PropertiesAction.class),
        };
    }
    
    protected MultiDataObject createMultiObject(FileObject primaryFile) throws DataObjectExistsException, IOException {                
        return new ZumlDataObject(primaryFile, this);
    }        
}
