package eduni.inf.pepa.pepamodel;

import eduni.inf.pepa.pepamodel.other.PepaDefinition;
import eduni.inf.pepa.pepamodel.process.PepaProcess;
import eduni.inf.pepa.pepamodel.rate.PepaRate;
import eduni.inf.pepa.pepamodel.rate.PepaSpecifiedRate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;




/** <p>This class represents a single PEPA model.</p>
 *  Containing:<br>
 *  <ol><li>the defined rate variables as a hash table
 *  <li>a list of definitions
 *  <li>the system equation
 *  </ol>
 *
 *
 */
public class PepaModel{
    private List rates;
    private List definitions;
    private PepaProcess systemEquation;
    
    public PepaModel(){
        this.rates = new ArrayList();
        this.definitions = new ArrayList();
    }
    
    public void addDefinition(PepaDefinition definition){
        this.definitions.add(definition);
    }
    public List getDefinitions(){
        return new ArrayList(definitions);
    }
    /** Convenience method for exporting the definitions as a HashMap */
    public HashMap getDefinitionsHashMap(){
        PepaDefinition d;
        HashMap definitionsHashMap = new HashMap(definitions.size());
        for(Iterator i = definitions.iterator();i.hasNext();){
            d = (PepaDefinition)i.next();
            definitionsHashMap.put(d.getVariable(),d.getDefinition());
        }
        return definitionsHashMap;
    }
    public void addRate(PepaRate rate){
        rates.add(rate);
    }
    public List getRates(){
        return new ArrayList(rates);
    }
    
    public void setSystemEquation(PepaProcess systemEquation){
        this.systemEquation = systemEquation; 
    }
    
    public PepaProcess getSystemEquation(){
        return this.systemEquation;
    }
    
    /** Checks if this PEPA model contains
     *  a rate with the given String as identifier
     *
     * @return the rate if it is found, and null otherwise
     */
    public PepaRate containsRate(String rate){
        for(Iterator i = rates.iterator();i.hasNext();){
            Object o;
            if((o=i.next()) instanceof PepaSpecifiedRate){                
                PepaSpecifiedRate next =    (PepaSpecifiedRate)o;
                if(next.getIdentifier().equals(rate)){                    
                    return next;
                }
            }
        }
        return null;        
    }
    
    public String toString(){
        StringBuffer b = new StringBuffer();
        String lineSeparator = System.getProperty("line.separator");
        for(Iterator i = rates.iterator();i.hasNext();){
            b.append(((PepaRate)i.next()).toString()+lineSeparator);
        }
        for(Iterator i = definitions.iterator();i.hasNext();){
            b.append(((PepaDefinition)i.next()).toString()+lineSeparator);
        }
        b.append(this.systemEquation.toString()+lineSeparator);
        return b.toString();
        
    }
}
