
package eduni.inf.pepa.pepamodel;

import eduni.inf.pepa.pepamodel.other.OtherTest;
import eduni.inf.pepa.pepamodel.other.PepaAction;
import eduni.inf.pepa.pepamodel.other.PepaActivity;
import eduni.inf.pepa.pepamodel.process.ProcessTest;
import eduni.inf.pepa.pepamodel.rate.PepaNumericRate;
import eduni.inf.pepa.pepamodel.rate.RateTest;
import junit.extensions.TestSetup;
import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

/** JUnit tests for <code>{@link eduni.inf.pepa.pepamodel}</code>.
 *
 * <p>This class servers as a super class for JUnit tests, and
 *    hence provides a set of convenience methods.
 * </p>
 *
 *
 * @author  N.V.Haenel
 * @since 0.9.5
 */
public class PepaModelTest extends TestCase{
    
    // here is the fixture
    public static PepaAction actionX, actionY, actionZ, actionW;
    public static PepaActivity activityX, activityY, activityZ, activityW;
    public static PepaNumericRate r = new PepaNumericRate(1.0);
    
    public PepaModelTest(String testName) {
        super(testName);
    }
    
    public static void main(String[] args) {        
        TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite();
        // add the static inner classes here
        suite.addTest(OtherTest.suite());
        suite.addTest(ProcessTest.suite());
        suite.addTest(RateTest.suite());
        
        
        TestSetup wrapper = new TestSetup(suite) {
            protected void setUp() { oneTimeSetUp(); }
            protected void tearDown() { oneTimeTearDown(); }
        };
        return wrapper;
    }
    
    public static void oneTimeSetUp() {
    }
    
    public static void oneTimeTearDown() {
        
    }
    
    /** signum(integer x) gives the value of the sign of x.
     *
     * @return -1 if negative, 1 if positive and 0 if zero
     *
     */
    public static int signum(int x){
        if(x<0) return -1;
        else if(x==0) return 0;
        else return 1;
    }
    
    /** Generalised method to test for the equals contract.
     *
     *  <p> The parameters <code>x</code>,<code>y</code> and <code>z</code>
     *      should all be equal. </p>
     *
     *  @see Object#equals
     */
    
    public void testEquals(Object x, Object y,Object z){
        // test reflexivity
        Assert.assertTrue("expected equals to be reflexive",x.equals(x));
        //test symetry
        Assert.assertTrue("expected equals to be symetric ",x.equals(y) && y.equals(x));
        //test transitivity
        Assert.assertTrue("expected equals to be transitive ",
        x.equals(y) && y.equals(z) && x.equals(z));
        // test nuon-nullity
        Assert.assertFalse("expected equals to be false for null",x.equals(null));
    }
    
    /** Generalised method to test the compareTo contract
     *
     *  <p> The parameter <code>x</code> should be greater than
     * <code>y</code> should be greater than <code>z</code>. </p>
     *
     * <p>The latter part of the contract is tested with
     *    <code>testCompareTo(Comparable x, Comparable y, Comparable z, Comparable w)</code>.
     * </p>
     *
     * @see java.lang.Comparable#compareTo
     * @see testCompareTo(Comparable x, Comparable y, Comparable z, Comparable w)
     */
    public void testCompareTo(Comparable x, Comparable y, Comparable z){
        // test that sgn(x.compareTo(y)) == -(sgn(y.compareTo(x)));
        Assert.assertTrue("expected sgn(x.comapareTo(y)) == -sgn(y.compareTo(x))",
        PepaModelTest.signum(x.compareTo(y)) == -PepaModelTest.signum(y.compareTo(x)));
        
        try{
            x.compareTo(new String("test"));
            Assert.fail("Expected ClassCastException when comparing "+x.getClass().getName()+" to String");
        }catch(ClassCastException cce){  }
       
            if (x instanceof String) 
                Assert.fail("Expected ClassCastException when comparing String to "+x.getClass().getName());
        
        //testing transitivity
        Assert.assertTrue("Expecting compareTo to be transitive",y.compareTo(x) > 0
        && z.compareTo(y) > 0 && z.compareTo(x)>0);
    }
    
    /** Generalised method to test the last requirement of the compareTo() contract */
    public void testCompareTo(Comparable x, Comparable y, Comparable z, Comparable w){
        // testing that if x.compareTo(y) ==0
        // sgn(x.compareTo(z)) == sgn(x.compareTo(Z))
        
        Assert.assertTrue("Expecting compareTo to be distributive(i think)",(x.compareTo(y) == 0)
        && (PepaModelTest.signum(x.compareTo(z)) == PepaModelTest.signum(y.compareTo(z)))
        && (PepaModelTest.signum(x.compareTo(w)) == PepaModelTest.signum(y.compareTo(w))));
    }
    
    /** Generalised method to test that equals() and compareTo() are consistent.
     *  <p>That is, x.equals(y) == true iff x.compareTo(y) == 0.</p>
     *
     * @see java.lang.Comparable
     */
    public void testEqualsCompareToConsistency(Comparable x, Comparable y){
        Assert.assertTrue("Expecting equals and compareTo to be consistent",
        (x.compareTo(y) == 0) && x.equals(y));
    }
}
