package eduni.inf.pepa.pepamodel.other;

import eduni.inf.pepa.pepamodel.PepaModelTest;
import eduni.inf.pepa.pepamodel.process.PepaVariable;
import eduni.inf.pepa.pepamodel.rate.PepaRate;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

/** Test cases for package <code>{@link eduni.inf.pepa.pepamodel.other}</code>.
 *
 * @author N.V.Haenel
 * @since 0.9.5
 */
public class OtherTest extends PepaModelTest {
    
    public OtherTest(java.lang.String testName) { super(testName); }
    
    public static void main(String[] args){
        TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite();
        
        suite.addTest(new TestSuite(TestPepaAction.class));
        suite.addTest(new TestSuite(TestPepaActivity.class));
        suite.addTest(new TestSuite(TestPepaDerivative.class));
        
        return suite;
    }    
    /**JUnit Test Case for <code>{@link PepaAction}</code> . */
    public static class TestPepaAction extends OtherTest{
        
        public TestPepaAction(String testName){
            super(testName);
        }
        
        /** Tests the equals() contract */
        public void testEquals(){
            actionX = new PepaAction("action");
            actionY = new PepaAction("action");
            actionZ = new PepaAction("action");
            
            System.out.println("Testing PepaAction equals()");
            //testing equals contract
            testEquals(actionX,actionY,actionZ);
        }
        
        /** Tests the compareTo() contract */
        public void testCompareTo(){
            System.out.println("Testing PepaAction compareTo()");
            
            actionX = new PepaAction("actionX");
            actionY = new PepaAction("actionY");
            actionZ = new PepaAction("actionZ");
            
            testCompareTo(actionX,actionY,actionZ);
            
            actionX = new PepaAction("action");
            actionY = new PepaAction("action");
            actionZ = new PepaAction("actionZ");
            actionW = new PepaAction("action");
            
            testCompareTo(actionX,actionY,actionZ,actionW);
        }
        
        /** Tests consistency of equals() and compareTo() */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaAction for equals() and compareTo() consistency");
            actionX = new PepaAction("action");
            actionY = new PepaAction("action");
            testEqualsCompareToConsistency(actionX,actionY);
        }
        
        
    }
    
    /**JUnit Test Case for <code>{@link eduni.inf.pepa.pepamodel.other.PepaActivity}</code>.*/
    public static class TestPepaActivity extends OtherTest{
        
        public TestPepaActivity(String testName){ super(testName); }
        /** Tests the equals() contract */
        public void testEquals(){
            System.out.println("Testing PepaActivity equals()");
            actionX = new PepaAction("action");
            actionY = new PepaAction("action");
            actionZ = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            activityY = new PepaActivity(actionY, r);
            activityZ = new PepaActivity(actionZ, r);
            
            testEquals(activityX,activityY,activityZ);
            
        }
        /** Tests the compareTo() contract */
        public void testCompareTo(){
            System.out.println("Testing PepaActivity compareTo()");
            
            actionX = new PepaAction("actionX");
            actionY = new PepaAction("actionY");
            actionZ = new PepaAction("actionZ");
            
            activityX = new PepaActivity(actionX, r);
            activityY = new PepaActivity(actionY, r);
            activityZ = new PepaActivity(actionZ, r);
            
            testCompareTo(activityX,activityY,activityZ);
            
            actionX = new PepaAction("action");
            actionY = new PepaAction("action");
            actionZ = new PepaAction("actionZ");
            actionW = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            activityY = new PepaActivity(actionY, r);
            activityZ = new PepaActivity(actionZ, r);
            activityW = new PepaActivity(actionW, r);
            
            testCompareTo(activityX,activityY,activityZ,activityW);
            
        }
        
        /** Tests equals() and compareTo() for consistency */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaActivity for equals() and compareTo() consistency");
            
            actionX = new PepaAction("action");
            actionY = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            activityY = new PepaActivity(actionY, r);
            
            this.testEqualsCompareToConsistency(activityX,activityY);
        }
    }
    
    /**JUnit Test Case for <code>{@link eduni.inf.pepa.pepamodel.other.PepaDerivative}</code>.
     *
     * <p>Importantly this does not test equals() as the compositional nature of the
     *    <code>TestPepaDerivative</code> class dictates that the composite objects
     *     should be tested on their own. This test case simply tests the
     *     <code>compareTo()</code> method.
     *
     */
    public static class TestPepaDerivative extends OtherTest{
        
        public TestPepaDerivative(String testName){ super(testName); }
        
        /** Tests the compareTo() contract */
        public void testCompareTo(){
            System.out.println("Testing PepaDerivative compareTo()");
            
            PepaAction[] actions = new PepaAction[1];
            PepaRate[] rates = new PepaRate[1];
            PepaVariable[] vars = new PepaVariable[1];
            
            PepaDerivative derivX = new PepaDerivative(new PepaVariable("proX"), actions , rates,vars);
            PepaDerivative derivY = new PepaDerivative(new PepaVariable("proY"), actions , rates,vars);
            PepaDerivative derivZ = new PepaDerivative(new PepaVariable("proZ"), actions , rates,vars);
            
            testCompareTo(derivX,derivY,derivZ);
            
            derivX = new PepaDerivative(new PepaVariable("pro"), actions , rates,vars);
            derivY = new PepaDerivative(new PepaVariable("pro"), actions , rates,vars);
            derivZ = new PepaDerivative(new PepaVariable("proZ"), actions , rates,vars);
            PepaDerivative derivW = new PepaDerivative(new PepaVariable("pro"), actions , rates,vars);
            
            testCompareTo(derivX,derivY,derivZ,derivW);
        }
    }
}
