package eduni.inf.pepa.pepamodel.process;

import eduni.inf.pepa.pepamodel.other.ActionSet;
import eduni.inf.pepa.pepamodel.visitors.PepaProcessVisitor;


/** Represents PEPA Cooperation objects.
 *
 *
 * @see java.lang.Comparable
 * @see eduni.inf.pepa.pepamodel.other.ActionSet
 * @see eduni.inf.pepa.pepamodel.other.PepaAction
 *
 * @author N.V.Haenel
 * @since 0.9.5
 */

public class PepaCooperation extends PepaProcess implements PepaProcessVisitable {
    private PepaProcess process1;
    private PepaProcess process2;    
    private ActionSet coopSet;
        
    public PepaCooperation(PepaProcess process1, PepaProcess process2, ActionSet coopSet){
        this.process1 = process1;
        this.process2 = process2;
        this.coopSet = coopSet;
    }
    
    public PepaProcess getProcess1(){
        return process1;
    }
    public PepaProcess getProcess2(){
        return process2;
    }
    public void setProcess1(PepaProcess p){
        this.process1 = p;
    }
    public void setProcess2(PepaProcess p){
        this.process2 = p;
    }
    
    public ActionSet getCoopSet(){
        return coopSet;
    }
    
    
    public void accept(PepaProcessVisitor visitor){
        visitor.visit(this);
    }
    
    public boolean equals(Object o){
        return (o instanceof PepaCooperation)
        && ((process1.equals(((PepaCooperation)o).getProcess1()) && process2.equals(((PepaCooperation)o).getProcess2()))
        || (process2.equals(((PepaCooperation)o).getProcess1()) && process1.equals(((PepaCooperation)o).getProcess2())))
        && coopSet.equals(((PepaCooperation)o).getCoopSet());
    }
    public int hashCode(){
        int result = 17;
        result = 37*result + process1.hashCode();
        result = 37*result + process2.hashCode();
        result = 37*result + coopSet.hashCode();
        return result;
    }
    
    public String toString(){
        return "("+process1.toString()+(coopSet.isEmpty()?"||":"<"+coopSet.toString()+">")+process2.toString()+")";
    }
    
    public int compareTo(Object o) {
        if(o instanceof PepaCooperation){
            PepaCooperation that = (PepaCooperation)o;
            int k;
            if((k = this.getProcess1().compareTo(that.getProcess1()))!=0){
                return k;
            }else if ((k = this.getProcess2().compareTo(that.getProcess2()))!=0){
                return k;
            }else
                return this.getCoopSet().compareTo(that.getCoopSet());
            
       }else return super.compareTo(o);
    }
    
    
}
