package eduni.inf.pepa.pepamodel.process;

import eduni.inf.pepa.pepamodel.PepaModelTest;
import eduni.inf.pepa.pepamodel.other.ActionSet;
import eduni.inf.pepa.pepamodel.other.PepaAction;
import eduni.inf.pepa.pepamodel.other.PepaActivity;
import java.util.Arrays;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

/** JUnit tests for <code>{@link eduni.inf.pepa.pepamodel.process}</code>.
 *
 * @author  N.V.Haenel
 * @since 0.9.5
 */
public class ProcessTest extends PepaModelTest{
    
    PepaVariable varX, varY, varZ, varW;
    PepaPrefix prefixX, prefixY, prefixZ, prefixW;
    PepaChoice choiceX, choiceY, choiceZ, choiceW;
    PepaCooperation coopX, coopY, coopZ, coopW;
    PepaHide hideX, hideY, hideZ, hideW;
    
    public ProcessTest(String testName) {
        super(testName);
    }
    public static void main(String[] args) {
        TestRunner.run(suite());
    }
    public static Test suite() {
        TestSuite suite = new TestSuite();
        // add the static inner classes here
        suite.addTest(new TestSuite(TestPepaVariable.class));
        suite.addTest(new TestSuite(TestPepaPrefix.class));
        suite.addTest(new TestSuite(TestPepaChoice.class));
        suite.addTest(new TestSuite(TestPepaCooperation.class));
        suite.addTest(new TestSuite(TestPepaHide.class));
        suite.addTest(new TestSuite(TestPepaProcess.class));
        
        return suite;
    }
    
    
    /** JUnit tests for <code>{@link eduni.inf.pepa.pepamodel.process.PepaVariable}</code>. */
    public static class TestPepaVariable extends ProcessTest{
        
        public TestPepaVariable(String testName){
            super(testName);
        }
        /** Tests the equals contract */
        public void testEquals(){
            System.out.println("Testing PepaVariable equals()");
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("var");
            
            testEquals(varX,varY,varZ);
        }
        
        /** Tests the compareTo() contract */
        public void testCompareTo(){
            System.out.println("Testing PepaVariable compareTo()");
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            varZ = new PepaVariable("varZ");
            
            testCompareTo(varX,varY,varZ);
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("varZ");
            varW = new PepaVariable("var");
            
            testCompareTo(varX,varY,varZ,varW);
        }
        /** Tests equals() and compareTo() for consistency */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaVariable for equals() and compareTo() consistency");
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varX");
            
            testEqualsCompareToConsistency(varX,varY);
        }
    }
    
    /** JUnit tests for <code>{@link eduni.inf.pepa.pepamodel.process.PepaPrefix}</code>. */
    public static class TestPepaPrefix extends ProcessTest{
        
        public TestPepaPrefix(String testName){
            super(testName);
        }
        
        /** Tests the equals contract */
        public void testEquals(){
            System.out.println("Testing PepaPrefix equals()");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("var");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            prefixZ = new PepaPrefix(activityX, varZ);
            
            testEquals(prefixX,prefixY,prefixZ);
        }
        
        /** Tests the compareTo contact */
        public void testCompareTo(){
            System.out.println("Testing PepaPrefix compareTo()");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            varZ = new PepaVariable("varZ");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            prefixZ = new PepaPrefix(activityX, varZ);
            
            testCompareTo(prefixX,prefixY,prefixZ);
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("varZ");
            varW = new PepaVariable("var");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            prefixZ = new PepaPrefix(activityX, varZ);
            prefixW = new PepaPrefix(activityX, varW);
            
            testCompareTo(prefixX,prefixY,prefixZ,prefixW);
        }
        
        /** Tests equals() and compareTo() for consistency */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaPrefix for equals() and compareTo() consistency");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varX");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            
            testEqualsCompareToConsistency(prefixX,prefixY);
        }
    }
    
    /** JUnit test for <code>{@link eduni.inf.pepa.pepamodel.process.PepaChoice}</code>. */
    public static class TestPepaChoice extends ProcessTest{
        
        public TestPepaChoice(String testName){
            super(testName);
            
        }
        public void testEquals(){
            System.out.println("Testing PepaChoice equals()");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            varZ = new PepaVariable("varZ");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            prefixZ = new PepaPrefix(activityX, varZ);
            
            choiceX = new PepaChoice( Arrays.asList(
            new PepaProcess[]{ prefixX,prefixY,prefixZ }));
            
            choiceY = new PepaChoice( Arrays.asList(
            new PepaProcess[]{ prefixZ,prefixY,prefixX }));
            
            choiceZ = new PepaChoice( Arrays.asList(
            new PepaProcess[]{ prefixX,prefixZ,prefixY }));
            
            testEquals(choiceX,choiceY,choiceZ);
        }
        
        public void testCompareTo(){
            System.out.println("Testing PepaChoice compareTo()");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            varZ = new PepaVariable("varZ");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            prefixZ = new PepaPrefix(activityX, varZ);
            
            choiceX = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixX }));
            
            choiceY = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixY }));
            
            choiceZ = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixZ }));
            
            testCompareTo(choiceX,choiceY,choiceZ);
            
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("varZ");
            varW = new PepaVariable("var");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            prefixZ = new PepaPrefix(activityX, varZ);
            prefixW = new PepaPrefix(activityX, varW);
            
            choiceX = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixX }));
            choiceY = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixY }));
            choiceZ = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixZ }));
            choiceW = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixW }));
            
            testCompareTo(choiceX,choiceY,choiceZ, choiceZ);
        }
        
        /** Testing equals() and compareTo() for consistency */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaChoice for equals() and compareTo() consistency");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varX");
            
            prefixX = new PepaPrefix(activityX,varX);
            prefixY = new PepaPrefix(activityX,varY);
            
            choiceX = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixX }));
            choiceY = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixY }));
            
            testEqualsCompareToConsistency(choiceX, choiceY);
        }
        
    }
    
    /** JUnit test for <code>{@link eduni.inf.pepa.pepamodel.process.PepaHide}</code>. */
    public static class TestPepaHide extends ProcessTest{
        public TestPepaHide(String testName){
            super(testName);
        }
        
        /** Test the equals() contract */
        public void testEquals(){
            System.out.println("Testing PepaHide equals()");
            varX = new PepaVariable("var");
            
            ActionSet set = new ActionSet();
            
            hideX = new PepaHide(varX,set);
            hideY = new PepaHide(varX,set);
            hideZ = new PepaHide(varX,set);
            
            testEquals(hideX,hideY,hideZ);
        }
        /** Tests the compareTo() contract */
        public void testCompareTo(){
            System.out.println("Testing PepaHide compareTo()");
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            varZ = new PepaVariable("varZ");
            
            ActionSet set = new ActionSet();
            
            hideX = new PepaHide(varX,set);
            hideY = new PepaHide(varY,set);
            hideZ = new PepaHide(varZ,set);
            
            testCompareTo(hideX,hideY,hideZ);
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("varZ");
            varW = new PepaVariable("var");
            
            hideX = new PepaHide(varX,set);
            hideY = new PepaHide(varY,set);
            hideZ = new PepaHide(varZ,set);
            hideW = new PepaHide(varW,set);
            
            testCompareTo(hideX,hideY,hideZ,hideW);
        }
        
        /** Testing equals() and compareTo() for consistency */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaHide for equals() and compareTo() consistency");
            
            actionX = new PepaAction("action");
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            
            ActionSet set = new ActionSet();
            
            hideX = new PepaHide(varX,set);
            hideY = new PepaHide(varY,set);
            
            testEqualsCompareToConsistency(hideX,hideY);
        }
    }
    
    
    /** JUnit test for <code>{@link eduni.inf.pepa.pepamodel.process.PepaCooperation}</code>. */
    public static class TestPepaCooperation extends ProcessTest{
        public TestPepaCooperation(String testName){
            super(testName);
        }
        
        /** Test the equals() contract */
        public void testEquals(){
            System.out.println("Testing PepaCooperation equals()");
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            
            ActionSet set = new ActionSet();
            
            coopX = new PepaCooperation(varX,varY,set);
            coopY = new PepaCooperation(varX,varY,set);
            coopZ = new PepaCooperation(varX,varY,set);
            
            testEquals(coopX,coopY,coopZ);
            
        }
        /** Tests the compareTo() contract */
        public void testCompareTo(){
            System.out.println("Testing PepaCooperation compareTo()");
            
            varX = new PepaVariable("varX");
            varY = new PepaVariable("varY");
            varZ = new PepaVariable("varZ");
            
            ActionSet set = new ActionSet();
            
            coopX = new PepaCooperation(varX,varX,set);
            coopY = new PepaCooperation(varY,varY,set);
            coopZ = new PepaCooperation(varZ,varZ,set);
            
            testCompareTo(coopX, coopY, coopZ);
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            varZ = new PepaVariable("varZ");
            varW = new PepaVariable("var");
            
            // set still in scope
            
            coopX = new PepaCooperation(varX,varX,set);
            coopY = new PepaCooperation(varY,varY,set);
            coopZ = new PepaCooperation(varZ,varZ,set);
            coopW = new PepaCooperation(varW,varW,set);
            
            testCompareTo(coopX, coopY, coopZ, coopW);
        }
        
        /** Testing equals() and compareTo() for consistency */
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaCooperation for equals() and compareTo() consistency");
            
            actionX = new PepaAction("action");
            
            varX = new PepaVariable("var");
            varY = new PepaVariable("var");
            
            ActionSet set = new ActionSet();
            
            coopX = new PepaCooperation(varX,varX,set);
            coopY = new PepaCooperation(varY,varY,set);
            
            testEqualsCompareToConsistency(coopX,coopY);
        }
        
    }
    
    /** JUnit test for <code>{@link eduni.inf.pepa.pepamodel.process.PepaProcess}</code>. */
    public static class TestPepaProcess extends ProcessTest{
        public TestPepaProcess(String testName){
            super(testName);
        }
        
        /** Tests the relative oredering of PepaProcess objects.
         *
         * <p>The relative order is:
         * <code>
         * <ol><li>PepaVariable
         *     <li>PepaChoice
         *     <li>PepaPrefix
         *     <li>PepaHide
         *     <li>PepaCooperation
         * </ol>
         * </code>
         *
         * <p> Note also that there is no need to compare the Objects
         *     to themselves as this is done in the unit tests
         *     for the respective classes. Also note that checking for
         *     <code>ClassCastExceptions</code> when comparing to
         *     non PepaProcess classes is performed in the individual
         *     unit tests and is not replicated here.
         * </p>
         */
        public void testCompareTo(){
            System.out.println("Testing PepaProcess compareTo()");
            
            actionX = new PepaAction("action");
            
            activityX = new PepaActivity(actionX, r);
            
            varX = new PepaVariable("varX");
            
            prefixX = new PepaPrefix(activityX,varX);
            
            choiceX = new PepaChoice( Arrays.asList( new PepaProcess[]{ prefixX }));
            
            coopX = new PepaCooperation(varX,varX,new ActionSet());
            
            hideX = new PepaHide(varX, new ActionSet());
            
            // test PepaVariable
            
            assertEquals("PepaVariable should be less than PepaPrefix", varX.compareTo(prefixX), -1);
            assertEquals("PepaVariable should be less than PepaChoice", varX.compareTo(prefixX), -1);
            assertEquals("PepaVariable should be less than PepaCooperation", varX.compareTo(coopX), -1);
            assertEquals("PepaVariable should be less than PepaHide", varX.compareTo(hideX), -1);
            
            // test PepaPrefix
            
            assertEquals("PepaPrefix should be greater than PepaVariable", prefixX.compareTo(varX), 1);
            assertEquals("PepaPrefix should be less than PepaChoice", prefixX.compareTo(choiceX), -1);
            assertEquals("PepaPrefix should be less than PepaCooperation", prefixX.compareTo(coopX), -1);
            assertEquals("PepaPrefix should be less than PepaHide", prefixX.compareTo(hideX), -1);
            
            // test PepaChoice
            
            assertEquals("PepaChoice should be greater than PepaVariable", choiceX.compareTo(varX), 1);
            assertEquals("PepaChoice should be greater than PepaPrefix", choiceX.compareTo(prefixX), 1);
            assertEquals("PepaChoice should be less than PepaCooperation", choiceX.compareTo(coopX), -1);
            assertEquals("PepaChoice should be less than PepaHide", choiceX.compareTo(hideX), -1);
            
            // test PepaCooperation
            
            assertEquals("PepaCooperation shouold be greater than PepaVariable", coopX.compareTo(varX), 1);
            assertEquals("PepaCooperation shouold be greater than PepaPrefix", coopX.compareTo(prefixX), 1);
            assertEquals("PepaCooperation shouold be greater than PepaChoice", coopX.compareTo(choiceX), 1);
            assertEquals("PepaCooperation shouold be less than PepaHide", coopX.compareTo(hideX), -1);
            
            // test PepaHide
            
            assertEquals("PepaHide should be greater than PepaVariable", hideX.compareTo(varX), 1);
            assertEquals("PepaHide should be greater than PepaPrefix", hideX.compareTo(prefixX), 1);
            assertEquals("PepaHide should be greater than PepaChoice", hideX.compareTo(choiceX), 1);
            assertEquals("PepaHide shuold be greater than PepaCooperation", hideX.compareTo(coopX), 1);
        }
    }
}


