package eduni.inf.pepa.pepamodel.rate;

/** Superclass for PEPA rates 
 *
 *
 *
 * @author  N.V.Haenel
 * @since 0.9.5
 */
public abstract class PepaRate implements Comparable{
    public abstract double evaluate();
    
    /** Provides relative comparison of PepaRate objects.
     *  <p>The following "relative" ordering is enforced:
     *  <ol><li><code>PepaSpecifiedRate</code>
     *      <li><code>PepaNumericRate</code>
     *      <li><code>PepaRateExpression</code>
     *      <li><code>PepaTopRate</code>
     *      <li><code>PepaUndefinedRate</code>
     *  </ol></p>
     */
    public int compareTo(Object o){
        if(o instanceof PepaSpecifiedRate){
            if(this instanceof PepaSpecifiedRate)
                return 0;
            else if(this instanceof PepaNumericRate)
                return 1;
            else if(this instanceof PepaRateExpression)
                return 1;
            else if(this instanceof PepaTopRate)
                return 1;
            else if(this instanceof PepaUndefinedRate)
                return 1;
        }else if(o instanceof PepaNumericRate){
            if(this instanceof PepaSpecifiedRate)
                return -1;
            else if(this instanceof PepaNumericRate)
                return 0;
            else if(this instanceof PepaRateExpression)
                return 1;
            else if(this instanceof PepaTopRate)
                return 1;
            else if(this instanceof PepaUndefinedRate)
                return 1;
        }else if(o instanceof PepaRateExpression){
            if(this instanceof PepaSpecifiedRate)
                return -1;
            else if(this instanceof PepaNumericRate)
                return -1;
            else if(this instanceof PepaRateExpression)
                return 0;
            else if(this instanceof PepaTopRate)
                return 1;
            else if(this instanceof PepaUndefinedRate)
                return 1;
        }else if(o instanceof PepaTopRate){
            if(this instanceof PepaSpecifiedRate)
                return -1;
            else if(this instanceof PepaNumericRate)
                return -1;
            else if(this instanceof PepaRateExpression)
                return -1;
            else if(this instanceof PepaTopRate)
                return 0;
            else if(this instanceof PepaUndefinedRate)
                return 1;
        }else if(o instanceof PepaUndefinedRate){
            if(this instanceof PepaSpecifiedRate)
                return -1;
            else if(this instanceof PepaNumericRate)
                return -1;
            else if(this instanceof PepaRateExpression)
                return -1;
            else if(this instanceof PepaTopRate)
                return -1;
            else if(this instanceof PepaUndefinedRate)
                return 0;
        }
        
        throw new ClassCastException(o.getClass().getName());
    }
    
}

