package eduni.inf.pepa.pepamodel.rate;

/** Represents PEPA rate expression objects 
 *
 * @author  N.V.Haenel
 * @since 0.9.5
 */
public class PepaRateExpression extends PepaRate{
    PepaRate left;
    PepaRate right;
    int operator;
    public static final int ADD = 1;
    public static final int SUBTRACT = 2;
    public static final int MULTIPLY = 3;
    public static final int DIVIDE = 4;
    
    public PepaRateExpression(PepaRate left, PepaRate right, int operator){
        this.left = left;
        this.right = right;
        this.operator = operator;
    }
    public PepaRateExpression(PepaRate left, PepaRate right, String operator){
        this.left = left;
        this.right = right;
        this.operator = this.convertOperator(operator);
        
    }
    
    
    public PepaRate getLeftOperand(){
        return left;
    }
    public PepaRate getRightOperand(){
        return right;
    }
    
    public String getOperator(){
        return this.convertOperator(this.operator);         
    }
    
    public double evaluate(){
        switch(operator){
            case 1: return left.evaluate() + right.evaluate();
            case 2: return left.evaluate() - right.evaluate();
            case 3: return left.evaluate() * right.evaluate();
            case 4: return left.evaluate() / right.evaluate();
            default: throw new ArithmeticException("Operator not defined, int value:"+operator);
        }        
    }
    public int hashCode(){
        int result = 17;
        result = 37*result + left.hashCode();
        result = 37*result + right.hashCode();
        result = 37*result + operator;
        return result;
    }
    
    private String convertOperator(int operator){
        switch(operator){
            case 1: return "+";
            case 2: return "-";
            case 3: return "*";
            case 4: return "/";
            default: throw new IllegalArgumentException("Undefined operator index in PepaRateExpression: "+operator);
        }        
    }
    private int convertOperator(String operator){
            if(operator == "+") return ADD;
            if(operator == "-") return SUBTRACT;
            if(operator == "*") return MULTIPLY;
            if(operator == "/") return DIVIDE;
            throw new IllegalArgumentException("Undefined operator index in PepaRateExpression: "+operator);
               
    }
    
            
    public String toString(){
        return "("+left.toString()+convertOperator(operator)+right.toString()+")";
    }
    
     public boolean equals(Object o){
        return o instanceof PepaRateExpression
        && ((PepaRateExpression)o).getLeftOperand().equals(this.getLeftOperand())
        && ((PepaRateExpression)o).getRightOperand().equals(this.getRightOperand())
        && ((PepaRateExpression)o).getLeftOperand().equals(this.getLeftOperand());
     }
     
     public int compareTo(Object o){
         if(o instanceof PepaRateExpression){
             return Double.compare(this.evaluate(),((PepaRateExpression)o).evaluate());
         }else
             return super.compareTo(o);
     }
}

