package eduni.inf.pepa.pepamodel.rate;

/** Represents PEPA rates that are undefined.
 *
 * These rates are specified by a rate identifier but have no corresponding
 * rate definition.
 *
 * 
 * @author  N.V.Haenel
 * @since 0.9.5
 */
public class PepaUndefinedRate extends PepaRate{
    private String identifier;
    
    /** Creates a new instance of PepaUndefinedRate */
    public PepaUndefinedRate(String identifier) {
        this.identifier = identifier;
    }
    public String getIdentifier(){
        return this.identifier;                
    }
    
    public double evaluate(){
        throw new UnsupportedOperationException("PepaUndefinedRate does not support evaluate()");
    }
    
    public int hashCode(){
        int result = 17;
        result = 37*result + identifier.hashCode();                
        return result;                       
    }
    
    public String toString(){
        return "("+this.identifier+")";
    }
    
    public boolean equals(Object o){
        return o instanceof PepaUndefinedRate
        && ((PepaUndefinedRate)o).getIdentifier().equals(this.getIdentifier());
    }
    
    public int compareTo(Object o){
        if(o instanceof PepaUndefinedRate)
            return this.getIdentifier().compareTo(((PepaUndefinedRate)o).getIdentifier());
        else
            return super.compareTo(o);
            
    }
}