package eduni.inf.pepa.pepamodel.rate;

import eduni.inf.pepa.pepamodel.PepaModelTest;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

/** JUnit test for  <code>{@link eduni.inf.pepa.pepamodel.rate}</code>.
 *
 * @author  N.V.Haenel
 * @since 0.9.5
 */
public class RateTest extends PepaModelTest {
    
    PepaNumericRate numX, numY, numZ, numW;
    PepaSpecifiedRate specX, specY, specZ, specW;
    PepaUndefinedRate undefX, undefY, undefZ, undefW;
    PepaRateExpression expX, expY, expZ, expW;
    
    public RateTest(String testName) {
        super(testName);
    }
    
    public static void main(String[] args) {
        TestRunner.run(suite());
    }
    
    public static Test suite() {
        TestSuite suite = new TestSuite();
        // add the static inner classes here
        suite.addTest(new TestSuite(TestPepaNumericRate.class));
        suite.addTest(new TestSuite(TestPepaSpecifiedRate.class));
        suite.addTest(new TestSuite(TestPepaUndefinedRate.class));        
        //suite.addTest(new RateTest("testPepaTopRateSingleton"));                
        return suite;
    }
    
    public void testPepaTopRateSingleton(){
        System.out.println("Testing PepaTopRate for singleton");
        PepaTopRate r1 = PepaTopRate.getTop();
        PepaTopRate r2 = PepaTopRate.getTop();
        
        this.assertEquals("PepaTopRate should be a singleton value", r1, r2);
        this.assertSame("PepaTopRate should be a singleton reference", r1, r2);                        
    }
    
    
    /** JUnit test for  <code>{@link eduni.inf.pepa.pepamodel.rate.PepaNumericRate}</code>. */

    public static class TestPepaNumericRate extends RateTest{
        
        public TestPepaNumericRate(String testName){
            super(testName);
        }
        
        public void testEquals(){
            System.out.println("Testing PepaNumericRate equals()");
            
            numX = new PepaNumericRate(1.0);
            numY = new PepaNumericRate(1.0);
            numZ = new PepaNumericRate(1.0);
            
            testEquals(numX,numY,numZ);
        }
        
        public void testCompareTo(){
            System.out.println("Testing PepaNumericRate compareTo()");
            
            numX = new PepaNumericRate(1.0);
            numY = new PepaNumericRate(2.0);
            numZ = new PepaNumericRate(3.0);
            
            testCompareTo(numX,numY,numZ);
            
            numX = new PepaNumericRate(1.0);
            numY = new PepaNumericRate(1.0);
            numZ = new PepaNumericRate(1.1);
            numW = new PepaNumericRate(1.0);
            
            testCompareTo(numX,numY,numZ,numW);
        }
        
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaNumericRate for equals() and compareTo() consistency");
            
            numX = new PepaNumericRate(1.0);
            numY = new PepaNumericRate(1.0);
            
            testEqualsCompareToConsistency(numX,numY);
        }
        
    }
    
    /** JUnit test for  <code>{@link eduni.inf.pepa.pepamodel.rate.PepaSpecifiedRate}</code>. */

    public static class TestPepaSpecifiedRate extends RateTest{
        public TestPepaSpecifiedRate(String testName){
            super(testName);
        }
        
        public void testEquals(){
            System.out.println("Testing PepaSpecifiedRate equals()");
            
            numX = new PepaNumericRate(1.0);
                                                
            specX = new PepaSpecifiedRate("rate",numX);
            specY = new PepaSpecifiedRate("rate",numX);
            specZ = new PepaSpecifiedRate("rate",numX);
            
            testEquals(specX, specY, specZ);
        }
        
        public void testCompareTo(){
            System.out.println("Testing PepaSpecifiedRate compareTo()");
            
            numX = new PepaNumericRate(1.0);
                        
            specX = new PepaSpecifiedRate("rateX",numX);
            specY = new PepaSpecifiedRate("rateY",numX);
            specZ = new PepaSpecifiedRate("rateZ",numX);
            
            testCompareTo(specX,specY,specZ);
            
            numX = new PepaNumericRate(1.0);
            numY = new PepaNumericRate(2.0);
            numZ = new PepaNumericRate(3.0);
                        
            specX = new PepaSpecifiedRate("rate",numX);
            specY = new PepaSpecifiedRate("rate",numY);
            specZ = new PepaSpecifiedRate("rate",numZ);
            
            testCompareTo(specX,specY,specZ);
            
            numX = new PepaNumericRate(1.0);
            numY = new PepaNumericRate(1.0);
            numZ = new PepaNumericRate(1.1);
            numW = new PepaNumericRate(1.0);
                                    
            specX = new PepaSpecifiedRate("rate",numX);
            specY = new PepaSpecifiedRate("rate",numY);
            specZ = new PepaSpecifiedRate("rate",numZ);
            specW = new PepaSpecifiedRate("rate",numW);
            
            testCompareTo(specX,specY,specZ,specW);
        }
        
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaNumericRate for equals() and compareTo() consistency");
            
            numX = new PepaNumericRate(1.0);
                        
            specX = new PepaSpecifiedRate("rate",numX);
            specY = new PepaSpecifiedRate("rate",numX);
            
            testEqualsCompareToConsistency(specX,specY);
        }
    }
    
    /** JUnit test for  <code>{@link eduni.inf.pepa.pepamodel.rate.PepaUndefinedRate}</code>. */

    public static class TestPepaUndefinedRate extends RateTest{
        
        public TestPepaUndefinedRate(String testName){
            super(testName);
        }
        
        public void testEquals(){
            System.out.println("Testing PepaUndefinedRate equals()");
            
            undefX = new PepaUndefinedRate("undef");
            undefY = new PepaUndefinedRate("undef");
            undefZ = new PepaUndefinedRate("undef");
            
            testEquals(undefX, undefY, undefZ);
        }
        
        public void testCompareTo(){
            System.out.println("Testing PepaUndefinedRate compareTo()");
            
            undefX = new PepaUndefinedRate("undefX");
            undefY = new PepaUndefinedRate("undefY");
            undefZ = new PepaUndefinedRate("undefZ");
            
            testCompareTo(undefX,undefY,undefZ);
            
            undefX = new PepaUndefinedRate("undef");
            undefY = new PepaUndefinedRate("undef");
            undefZ = new PepaUndefinedRate("undefZ");
            undefW = new PepaUndefinedRate("undef");
            
            testCompareTo(undefX,undefY,undefZ,undefW);
        }
        
        public void testEqualsCompareToConsistency(){
            System.out.println("Testing PepaUndefinedRate for equals() and compareTo() consistency");
            
            undefX = new PepaUndefinedRate("undefX");
            undefY = new PepaUndefinedRate("undefX");
            
            testEqualsCompareToConsistency(undefX,undefY);
        }
        
        public void testEvaluateThrowsException(){
            System.out.println("Testing PepaUndefinedRate evaluate()");
            undefX = new PepaUndefinedRate("undefX");            
            try{
                undefX.evaluate();
                fail("PepaUndefinedRate should have throwen an UnsupportedOperationException when invoking evaluate()");
            }catch(UnsupportedOperationException uoe){  }                        
        }
    }
    
    /** JUnit test for  <code>{@link eduni.inf.pepa.pepamodel.rate.PepaNumericRate}</code>. */

    public static class TestPepaRateExpression extends RateTest{
        public TestPepaRateExpression(String testName){
            super(testName);
        }
        
        
        
    }
}
