package eduni.inf.pepa.pepamodel.visitors;

import eduni.inf.pepa.pepamodel.process.PepaProcessVisitable;
import java.lang.reflect.Method;


/** <p>The base class for a reflective visitor pattern. Provides the basic
 *  <code>visit(Object o)</code> method, that uses refelction and the 
 *  name of the visitable class to establish what method to invoke.</p>
 *
 *  <p> If the name of the class to be visited is <code>Foo</code>.
 *  This class will look for the method <code>visitFoo(Foo foo)</code>
 *  in the subclasses and invoke this if it exists. If no corresponding method
 *  can be found the <code>defaultVisit(Object o)</code> method is invoked.
 *  The implementing subclass may either override the <code>defaultVisit(Object o)
 *  </code> method to throw an exception for unexpected classes,
 *  or simply do nothing, visitng only those classes we are interested in
 *  and ignoring the rest.
 *  
 * @author N.V.Haenel
 * @since  0.9.5
 *
 *
 */
public abstract class AbstractPepaProcessVisitor implements PepaProcessVisitor{
    
    void defaultVisit(Object o){
       System.out.println("default");
    }
    
    public void visit(Object o){      
        
      // First create the name of the method to invoke.
      String methodName = o.getClass().getName();
      methodName = "visit"+
                   methodName.substring(methodName.lastIndexOf('.')+1);      
        try {
             // Get the method visitFoo(Foo foo)
            Method m = getClass().getMethod(methodName,
                new Class[] { o.getClass() });
            // Try to invoke visitFoo(Foo foo)                
            m.invoke(this, new Object[] { o });
        } catch (NoSuchMethodException nme) {                        
            defaultVisit(o);            
        } catch (Exception e){
             // a more serious flaw somewhere.
            // consider throwing an error          
        }                   
    } 
    // this method isn't needed
    void callAccept(PepaProcessVisitable visitable) {
        visitable.accept(this);
    }
}
