package eduni.inf.pepa.pepamodel.visitors;

import eduni.inf.nbmodules.pepa.editor.parser.PepaParser;
import eduni.inf.pepa.pepamodel.PepaModel;
import eduni.inf.pepa.pepamodel.other.DerivativeSet;
import eduni.inf.pepa.pepamodel.statespace.StateSpaceExplorer;
import eduni.inf.pepa.pepamodel.visitors.DerivativeFinder;
import eduni.inf.pepa.pepamodel.visitors.ModelNormaliser;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;



/** JUnit tests for <code>eduni.inf.pepa.pepamodel.visitors.*</code>.
 */

public class VisitorsTest extends TestCase{
    /** The initial model */
    public static PepaModel model;
    /** The normalised model */
    public static PepaModel normed;
    /** The file for the model */
    public static File modelfile;
    /** The solution file */
    public static File solutionfile;
    /** The contents of the solution file */
    public static String solution;
    
    public VisitorsTest(String testName) {
        super(testName);
    }
    
    public static void main(String[] args) {
        modelfile = new File(args[0]);
        solutionfile = new File(args[1]);
        TestRunner.run(new TestSuite(NormaliserTest.class));
    }
    
    
    
    
    /** Tesst for eduni.inf.pepa.visitors.ModelNormaliser */
    public static class NormaliserTest extends VisitorsTest{
        public NormaliserTest(String testName){
            super(testName);
        }
        public void testNormliser(){
            try{
                PepaParser p = new PepaParser(new FileInputStream(modelfile));
                p.parse();
                model = p.getModel();
                ModelNormaliser mn = new ModelNormaliser();
                normed = mn.normaliseModel(this.model);
                loadSolution(solutionfile);
                assertEquals(normed.toString(), solution);
            }catch(IOException ioe){
                ioe.printStackTrace(System.err);
            }
        }
    }
    
    
    
    
    private static void loadSolution(File f) throws IOException{
        try{
            // make the SAX parsers
            SAXParserFactory factory = SAXParserFactory.newInstance();
            SAXParser parser = factory.newSAXParser();
            // parse the rates.xml file
            parser.parse(f, new SolutionFileHandler());
            
        }catch(ParserConfigurationException pce){
            pce.printStackTrace(System.err);
        }catch(FactoryConfigurationError fce){
            fce.printStackTrace(System.err);
        }catch(SAXException se){
            se.printStackTrace(System.err);
        }catch(IOException ioe){
            ioe.printStackTrace(System.err);
        }       
    }
    
    /** Content Handler for the model.solution file */
    private static class SolutionFileHandler extends DefaultHandler implements ContentHandler{
        private boolean sol = false;
        
        public void startElement(String namespaceURI,
        String localName,
        String qName,
        Attributes attributes){
            if(qName.equals("Solution"))
                sol = true;
        }
        
        public void endElement(String namespaceURI,
        String localName,
        String qName){
            if(qName.equals("Solution"))
                sol = false;                        
        }
        public void characters(char ch[], int start, int end){
            if(sol)
                solution = new String(ch, start, end);
        }
        
    }
    
}
